#/usr/bin/env python2.5

"""
mchronos.py

This is a simple PyGTK GUI applet intended to display astronomical
time information.  The application relies heavily upon a ported
version of the 'PyEphem' library created by Brandon Craig Rhodes.

More information about PyEphem is available at:

http://rhodesmill.org/pyephem/

More information about Python and PyGTK is available at:

http://www.python.org/
http://www.pygtk.org/

Copyright (C) 2008-2009 Darren Enns <dmenns1@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
USA.
"""

import sys
import gobject
import pango
import pygtk
pygtk.require('2.0')
import gtk
from gtk import gdk
import hildondesktop
import cairo
from datetime import datetime
import gobject
import math
import ephem
from ephem import stars
from ephem import cities
import os
import time
from math import pi, sin, cos, asin, atan2
import calendar
import ConfigParser
import string

try:
        import gpsbt
        gpsbt_flag = True
except ImportError:
        gpsbt_flag = False

VERSION = '0.40'

class mchronos(hildondesktop.HomeItem):
	def __init__(self):
		hildondesktop.HomeItem.__init__(self)

		self.local = local

		self.supports_alpha = supports_alpha

		self.graphics_style_all = graphics_style_all	
		self.graphics_style_pos = graphics_style_pos2
		self.graphics_style = self.graphics_style_all[self.graphics_style_pos] # test

		self.startup_long = startup_long
		self.startup_lat = startup_lat
		self.gps_mode = gps_mode
		self.gps_timeout_secs = gps_timeout_secs
		self.gps_time_flag = gps_time_flag
		self.text_flag = text_flag
		self.full_window_flag = full_window_flag
		self.clock_flag = clock_flag
		self.astro_flag = astro_flag
		self.astro_sign = astro_sign
		self.trails_flag = trails_flag
		self.sky_grid_flag = sky_grid_flag
		self.earth_grid_flag = earth_grid_flag
		self.moon_grid_flag = moon_grid_flag
		self.planet_mode = planet_mode
		self.planet_flag = planet_flag
		self.planet_label_flag = planet_label_flag
		self.star_mode = star_mode
		self.star_flag = star_flag
		self.star_label_flag = star_label_flag
		self.city_flag = city_flag
		self.city_label_flag = city_label_flag
		self.reset_flag = reset_flag
		self.gps_lock = gps_lock

		self.event_x = event_x
		self.event_y = event_y
		self.prev_event_x = prev_event_x
		self.prev_event_y = prev_event_y
		self.click_count = click_count

		self.set_resize_type(hildondesktop.HOME_ITEM_RESIZE_BOTH) # unnecessary

		self.resize_window()

    		self.connect("settings", self.display_menu)

		self.connect("expose-event", self.expose)
		self.connect("screen-changed", self.screen_changed)
		self.connect ("background", self.set_timer, False)
		self.connect ("foreground", self.set_timer, True)
		self.connect ("unrealize", self.unrealize)
		self.connect ("button_press_event", self.button_press_event)

		# set a timeout to update the clock, depending
		# on whether we are in the foreground or background
		self.timer = None
		self.set_timer(self, True)

		self.show_all()

    	def item_graphics_style_radio_toggled(self, widget, pos):
		self.graphics_style_pos = pos
		self.graphics_style = self.graphics_style_all[self.graphics_style_pos]

		self.resize_window()
		self.update()

		return

    	def item_text_flag_toggled(self, widget):
		if self.text_flag == "on":
			self.text_flag = "off"
			self.item_text_flag_check.set_active(False)
		else:
			self.text_flag = "on"
			self.item_text_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_full_window_flag_toggled(self, widget):
		if self.full_window_flag == "on":
			self.full_window_flag = "off"
			self.item_full_window_flag_check.set_active(False)
		else:
			self.full_window_flag = "on"
			self.item_full_window_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_clock_flag_toggled(self, widget):
		if self.clock_flag == "on":
			self.clock_flag = "off"
			self.item_clock_flag_check.set_active(False)
		else:
			self.clock_flag = "on"
			self.item_clock_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_astro_flag_toggled(self, widget):
		if self.astro_flag == "on":
			self.astro_flag = "off"
			self.astro_sign = 1
			self.item_astro_flag_check.set_active(False)
		else:
			self.astro_flag = "on"
			self.astro_sign = -1
			self.item_astro_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_trails_flag_toggled(self, widget):
		if self.trails_flag == "on":
			self.trails_flag = "off"
			self.item_trails_flag_check.set_active(False)
		else:
			self.trails_flag = "on"
			self.item_trails_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_sky_grid_flag_toggled(self, widget):
		if self.sky_grid_flag == "on":
			self.sky_grid_flag = "off"
			self.item_sky_grid_flag_check.set_active(False)
		else:
			self.sky_grid_flag = "on"
			self.item_sky_grid_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_earth_grid_flag_toggled(self, widget):
		if self.earth_grid_flag == "on":
			self.earth_grid_flag = "off"
			self.item_earth_grid_flag_check.set_active(False)
		else:
			self.earth_grid_flag = "on"
			self.item_earth_grid_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_moon_grid_flag_toggled(self, widget):
		if self.moon_grid_flag == "on":
			self.moon_grid_flag = "off"
			self.item_moon_grid_flag_check.set_active(False)
		else:
			self.moon_grid_flag = "on"
			self.item_moon_grid_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_reset_flag_toggled(self, widget):
		self.local.lat = self.startup_lat
		self.local.long = self.startup_long

		self.resize_window()
		self.update()

		return

    	def item_planet_label_flag_toggled(self, widget):
		if self.planet_label_flag == "on":
			self.planet_label_flag = "off"
			self.item_planet_label_flag_check.set_active(False)
		else:
			self.planet_label_flag = "on"
			self.item_planet_label_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_star_label_flag_toggled(self, widget):
		if self.star_label_flag == "on":
			self.star_label_flag = "off"
			self.item_star_label_flag_check.set_active(False)
		else:
			self.star_label_flag = "on"
			self.item_star_label_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_city_flag_toggled(self, widget):
		if self.city_flag == "on":
			self.city_flag = "off"
			self.item_city_flag_check.set_active(False)
		else:
			self.city_flag = "on"
			self.item_city_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_city_label_flag_toggled(self, widget):
		if self.city_label_flag == "on":
			self.city_label_flag = "off"
			self.item_city_label_flag_check.set_active(False)
		else:
			self.city_label_flag = "on"
			self.item_city_label_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

    	def item_gps_mode_radio_toggled(self, widget, state):
		self.gps_mode = state

		self.resize_window()
		self.update()

		return

    	def item_planet_mode_radio_toggled(self, widget, state):
		self.planet_mode = state

		if self.planet_mode in ["off", "on"]:
			self.planet_flag = self.planet_mode

		self.resize_window()
		self.update()

		return

    	def item_star_mode_radio_toggled(self, widget, state):
		self.star_mode = state

		if self.star_mode in ["off", "on"]:
			self.star_flag = self.star_mode

		self.resize_window()
		self.update()

		return

    	def item_gps_time_flag_toggled(self, widget):
		if self.gps_time_flag == "on":
			self.gps_time_flag = "off"
			self.item_gps_time_flag_check.set_active(False)
		else:
			self.gps_time_flag = "on"
			self.item_gps_time_flag_check.set_active(True)

		self.resize_window()
		self.update()

		return

  	def display_menu(self, widget, data=None):
		# create menus
		menu = gtk.Menu()
		mode = gtk.Menu()
		flag = gtk.Menu()
		gps = gtk.Menu()
		planet = gtk.Menu()
		star = gtk.Menu()
		location = gtk.Menu()

		# create menuitems
    		menuitem = gtk.MenuItem("MChronos")
    		modeitem = gtk.MenuItem("Modes")
    		flagitem = gtk.MenuItem("Flags")
    		gpsitem = gtk.MenuItem("GPS")
    		planetitem = gtk.MenuItem("Planets")
    		staritem = gtk.MenuItem("Stars")
    		locationitem = gtk.MenuItem("Locations")

		# create 'about' menuitem and attach to main menu
 		self.item_about = gtk.MenuItem ("About")
  		menu.append (self.item_about)

		# create 'save' menuitem and attach to main menu
 		self.item_save = gtk.MenuItem ("Save")
  		menu.append (self.item_save)

		# attach submenu 'mode' menuitem to main menu and create 'mode' radio buttons and append to 'mode' menu
		menu.append (modeitem)
                if len(self.graphics_style_all) >= 1:
  			self.item_graphics_style_radio1 = gtk.RadioMenuItem (None, self.graphics_style_all[0])
  			mode.append (self.item_graphics_style_radio1)
                if len(self.graphics_style_all) >= 2:
  			self.item_graphics_style_radio2 = gtk.RadioMenuItem (self.item_graphics_style_radio1, self.graphics_style_all[1])
  			mode.append (self.item_graphics_style_radio2)
                if len(self.graphics_style_all) >= 3:
  			self.item_graphics_style_radio3 = gtk.RadioMenuItem (self.item_graphics_style_radio2, self.graphics_style_all[2])
  			mode.append (self.item_graphics_style_radio3)
                if len(self.graphics_style_all) >= 4:
  			self.item_graphics_style_radio4 = gtk.RadioMenuItem (self.item_graphics_style_radio3, self.graphics_style_all[3])
  			mode.append (self.item_graphics_style_radio4)
                if len(self.graphics_style_all) >= 5:
  			self.item_graphics_style_radio5 = gtk.RadioMenuItem (self.item_graphics_style_radio4, self.graphics_style_all[4])
  			mode.append (self.item_graphics_style_radio5)
                if len(self.graphics_style_all) >= 6:
  			self.item_graphics_style_radio6 = gtk.RadioMenuItem (self.item_graphics_style_radio5, self.graphics_style_all[5])
  			mode.append (self.item_graphics_style_radio6)
                if len(self.graphics_style_all) >= 7:
  			self.item_graphics_style_radio7 = gtk.RadioMenuItem (self.item_graphics_style_radio6, self.graphics_style_all[6])
  			mode.append (self.item_graphics_style_radio7)
                if len(self.graphics_style_all) >= 8:
  			self.item_graphics_style_radio8 = gtk.RadioMenuItem (self.item_graphics_style_radio7, self.graphics_style_all[7])
  			mode.append (self.item_graphics_style_radio8)

  		#item_separator = gtk.SeparatorMenuItem () # does not appear to work i.e. show up on display
  		#menu.append (item_separator)

		# attach submenu 'flag' menuitem to main menu and create 'flag' check buttons and append to 'flag' menu
		menu.append (flagitem)
  		self.item_text_flag_check = gtk.CheckMenuItem ("Text")
  		#self.item_text_flag_check = gtk.CheckMenuItem (str(self.graphics_style_pos) + "dare444")
  		flag.append (self.item_text_flag_check)
  		self.item_full_window_flag_check = gtk.CheckMenuItem ("Full")
  		flag.append (self.item_full_window_flag_check)
  		self.item_clock_flag_check = gtk.CheckMenuItem ("Clock")
  		flag.append (self.item_clock_flag_check)
  		self.item_astro_flag_check = gtk.CheckMenuItem ("Astro")
  		flag.append (self.item_astro_flag_check)
  		self.item_trails_flag_check = gtk.CheckMenuItem ("Trails")
  		flag.append (self.item_trails_flag_check)
  		self.item_sky_grid_flag_check = gtk.CheckMenuItem ("Sky Grid")
  		flag.append (self.item_sky_grid_flag_check)
  		self.item_earth_grid_flag_check = gtk.CheckMenuItem ("Earth Grid")
  		flag.append (self.item_earth_grid_flag_check)
  		self.item_moon_grid_flag_check = gtk.CheckMenuItem ("Moon Grid")
  		flag.append (self.item_moon_grid_flag_check)

		# set check button style to radio instead of checkmark
		#self.item_text_flag_check.set_draw_as_radio(True)
		#self.item_full_window_flag_check.set_draw_as_radio(True)
		#self.item_clock_flag_check.set_draw_as_radio(True)
		#self.item_astro_flag_check.set_draw_as_radio(True)

		# attach submenu 'location' menuitem to main menu and create 'location' check buttons and append to 'location' menu
		menu.append (locationitem)
  		self.item_city_flag_check = gtk.CheckMenuItem ("Cities")
  		location.append (self.item_city_flag_check)
  		self.item_city_label_flag_check = gtk.CheckMenuItem ("Labels")
  		location.append (self.item_city_label_flag_check)
  		self.item_reset_flag_check = gtk.CheckMenuItem ("Reset")
  		location.append (self.item_reset_flag_check)

		# attach submenu 'gps' menuitem to main menu and create 'gps' radio buttons and append to 'gps' menu
		menu.append (gpsitem)
		self.item_gps_mode_radio1 = gtk.RadioMenuItem (None, "Off")
		gps.append (self.item_gps_mode_radio1)
		self.item_gps_mode_radio2 = gtk.RadioMenuItem (self.item_gps_mode_radio1, "Lock")
		gps.append (self.item_gps_mode_radio2)
		self.item_gps_mode_radio3 = gtk.RadioMenuItem (self.item_gps_mode_radio2, "On")
		gps.append (self.item_gps_mode_radio3)
  		self.item_gps_time_check = gtk.CheckMenuItem ("Time")
		gps.append (self.item_gps_time_check)

		# attach submenu 'planet' menuitem to main menu and create 'planet' radio buttons and append to 'planet' menu
		menu.append (planetitem)
		self.item_planet_mode_radio1 = gtk.RadioMenuItem (None, "Off")
		planet.append (self.item_planet_mode_radio1)
		self.item_planet_mode_radio2 = gtk.RadioMenuItem (self.item_planet_mode_radio1, "Auto")
		planet.append (self.item_planet_mode_radio2)
		self.item_planet_mode_radio3 = gtk.RadioMenuItem (self.item_planet_mode_radio2, "On")
		planet.append (self.item_planet_mode_radio3)
  		self.item_planet_label_flag_check = gtk.CheckMenuItem ("Labels")
  		planet.append (self.item_planet_label_flag_check)

		# attach submenu 'star' menuitem to main menu and create 'star' radio buttons and append to 'star' menu
		menu.append (staritem)
		self.item_star_mode_radio1 = gtk.RadioMenuItem (None, "Off")
		star.append (self.item_star_mode_radio1)
		self.item_star_mode_radio2 = gtk.RadioMenuItem (self.item_star_mode_radio1, "Auto")
		star.append (self.item_star_mode_radio2)
		self.item_star_mode_radio3 = gtk.RadioMenuItem (self.item_star_mode_radio2, "On")
		star.append (self.item_star_mode_radio3)
  		self.item_star_label_flag_check = gtk.CheckMenuItem ("Labels")
  		star.append (self.item_star_label_flag_check)

		# set 'mode' radio buttons to current values
		if self.graphics_style_pos == 0:
			self.item_graphics_style_radio1.set_active(True)
		elif self.graphics_style_pos == 1:
			self.item_graphics_style_radio2.set_active(True)
		elif self.graphics_style_pos == 2:
			self.item_graphics_style_radio3.set_active(True)
		elif self.graphics_style_pos == 3:
			self.item_graphics_style_radio4.set_active(True)
		elif self.graphics_style_pos == 4:
			self.item_graphics_style_radio5.set_active(True)
		elif self.graphics_style_pos == 5:
			self.item_graphics_style_radio6.set_active(True)
		elif self.graphics_style_pos == 6:
			self.item_graphics_style_radio7.set_active(True)
		elif self.graphics_style_pos == 7:
			self.item_graphics_style_radio8.set_active(True)

		# set 'gps' radio buttons to current values
		if self.gps_mode == "off":
			self.item_gps_mode_radio1.set_active(True)
		elif self.gps_mode == "lock":
			self.item_gps_mode_radio2.set_active(True)
		elif self.gps_mode == "on":
			self.item_gps_mode_radio3.set_active(True)

		# set 'planet' radio buttons to current values
		if self.planet_mode == "off":
			self.item_planet_mode_radio1.set_active(True)
		elif self.planet_mode == "auto":
			self.item_planet_mode_radio2.set_active(True)
		elif self.planet_mode == "on":
			self.item_planet_mode_radio3.set_active(True)

		# set 'star' radio buttons to current values
		if self.star_mode == "off":
			self.item_star_mode_radio1.set_active(True)
		elif self.star_mode == "auto":
			self.item_star_mode_radio2.set_active(True)
		elif self.star_mode == "on":
			self.item_star_mode_radio3.set_active(True)

		# set 'text' flag check button to current value
		if self.text_flag == "on":
			self.item_text_flag_check.set_active(True)
		else:
			self.item_text_flag_check.set_active(False)

		# set 'full window' flag check button to current value
		if self.full_window_flag == "on":
			self.item_full_window_flag_check.set_active(True)
		else:
			self.item_full_window_flag_check.set_active(False)

		if self.clock_flag == "on":
			self.item_clock_flag_check.set_active(True)
		else:
			self.item_clock_flag_check.set_active(False)

		# set 'astro' flag check button to current value
		if self.astro_flag == "on":
			self.item_astro_flag_check.set_active(True)
		else:
			self.item_astro_flag_check.set_active(False)

		# set 'trails' flag check button to current value
		if self.trails_flag == "on":
			self.item_trails_flag_check.set_active(True)
		else:
			self.item_trails_flag_check.set_active(False)

		# set 'sky grid' flag check button to current value
		if self.sky_grid_flag == "on":
			self.item_sky_grid_flag_check.set_active(True)
		else:
			self.item_sky_grid_flag_check.set_active(False)

		# set 'earth grid' flag check button to current value
		if self.earth_grid_flag == "on":
			self.item_earth_grid_flag_check.set_active(True)
		else:
			self.item_earth_grid_flag_check.set_active(False)

		# set 'moon grid' flag check button to current value
		if self.moon_grid_flag == "on":
			self.item_moon_grid_flag_check.set_active(True)
		else:
			self.item_moon_grid_flag_check.set_active(False)

		# set 'planet label' flag check button to current value
		if self.planet_label_flag == "on":
			self.item_planet_label_flag_check.set_active(True)
		else:
			self.item_planet_label_flag_check.set_active(False)

		# set 'star label' flag check button to current value
		if self.star_label_flag == "on":
			self.item_star_label_flag_check.set_active(True)
		else:
			self.item_star_label_flag_check.set_active(False)

		# set 'city' flag check button to current value
		if self.city_flag == "on":
			self.item_city_flag_check.set_active(True)
		else:
			self.item_city_flag_check.set_active(False)

		# set 'city label' flag check button to current value
		if self.city_label_flag == "on":
			self.item_city_label_flag_check.set_active(True)
		else:
			self.item_city_label_flag_check.set_active(False)

		# set 'reset location' flag check button to current value
		if self.reset_flag == "on":
			self.item_reset_flag_check.set_active(True)
		else:
			self.item_reset_flag_check.set_active(False)

		# connect 'mode' radio buttons to callback routine
		self.item_graphics_style_radio1.connect("activate", self.item_graphics_style_radio_toggled, 0)
		self.item_graphics_style_radio2.connect("activate", self.item_graphics_style_radio_toggled, 1)
		self.item_graphics_style_radio3.connect("activate", self.item_graphics_style_radio_toggled, 2)
		self.item_graphics_style_radio4.connect("activate", self.item_graphics_style_radio_toggled, 3)
		self.item_graphics_style_radio5.connect("activate", self.item_graphics_style_radio_toggled, 4)
		self.item_graphics_style_radio6.connect("activate", self.item_graphics_style_radio_toggled, 5)
		self.item_graphics_style_radio7.connect("activate", self.item_graphics_style_radio_toggled, 6)
		self.item_graphics_style_radio8.connect("activate", self.item_graphics_style_radio_toggled, 7)

		# connect 'flag' check buttons to callback routine
		self.item_text_flag_check.connect("toggled", self.item_text_flag_toggled)
		self.item_full_window_flag_check.connect("toggled", self.item_full_window_flag_toggled)
		self.item_clock_flag_check.connect("toggled", self.item_clock_flag_toggled)
		self.item_astro_flag_check.connect("toggled", self.item_astro_flag_toggled)
		self.item_trails_flag_check.connect("toggled", self.item_trails_flag_toggled)
		self.item_sky_grid_flag_check.connect("toggled", self.item_sky_grid_flag_toggled)
		self.item_earth_grid_flag_check.connect("toggled", self.item_earth_grid_flag_toggled)
		self.item_moon_grid_flag_check.connect("toggled", self.item_moon_grid_flag_toggled)
		self.item_reset_flag_check.connect("toggled", self.item_reset_flag_toggled)

		# connect 'location' check buttons to callback routine
		self.item_city_flag_check.connect("toggled", self.item_city_flag_toggled)
		self.item_city_label_flag_check.connect("toggled", self.item_city_label_flag_toggled)

		# connect 'gps' radio buttons to callback routine
		self.item_gps_mode_radio1.connect("activate", self.item_gps_mode_radio_toggled, "off")
		self.item_gps_mode_radio2.connect("activate", self.item_gps_mode_radio_toggled, "lock")
		self.item_gps_mode_radio3.connect("activate", self.item_gps_mode_radio_toggled, "on")
		self.item_gps_time_check.connect("toggled", self.item_gps_time_flag_toggled)

		# connect 'planet' radio buttons to callback routine
		self.item_planet_mode_radio1.connect("activate", self.item_planet_mode_radio_toggled, "off")
		self.item_planet_mode_radio2.connect("activate", self.item_planet_mode_radio_toggled, "auto")
		self.item_planet_mode_radio3.connect("activate", self.item_planet_mode_radio_toggled, "on")
		self.item_planet_label_flag_check.connect("toggled", self.item_planet_label_flag_toggled)

		# connect 'star' radio buttons to callback routine
		self.item_star_mode_radio1.connect("activate", self.item_star_mode_radio_toggled, "off")
		self.item_star_mode_radio2.connect("activate", self.item_star_mode_radio_toggled, "auto")
		self.item_star_mode_radio3.connect("activate", self.item_star_mode_radio_toggled, "on")
		self.item_star_label_flag_check.connect("toggled", self.item_star_label_flag_toggled)

		# connect 'about' menu to callback routine
		self.item_about.connect_object("activate", self.item_about_activated, "about")
		#item_lat.connect_object("activate", self.onLat, "latitude")

		# connect 'save' menu to callback routine
		self.item_save.connect_object("activate", self.item_save_activated, "save")

		# create submenus for menus
		menuitem.set_submenu(menu)
		modeitem.set_submenu(mode)
		flagitem.set_submenu(flag)
		gpsitem.set_submenu(gps)
		planetitem.set_submenu(planet)
		staritem.set_submenu(star)
		locationitem.set_submenu(location)

		menu.show_all()

    		return menuitem

    	def item_about_activated(self, widget):
        	dialog = gtk.AboutDialog()
        	dialog.set_name("MChronos")
		dialog.set_version(VERSION)
        	dialog.set_copyright("\302\251 Copyright 2008-2009 Darren Enns")
        	dialog.set_website("http://darethehair.googlepages.com/")
        	dialog.connect ("response", lambda d, r: d.destroy())
        	dialog.show()

		return

    	def item_save_activated(self, widget):
		ini_file = os.path.expanduser("~") + "/.mchronos/mchronos.ini"
		file = open(ini_file, "wb")
		file.write("[location]" + "\n")
		file.write("longitude: " + str(float(self.local.long) * 180/pi) + "\n")
		file.write("latitude: " + str(float(self.local.lat) * 180/pi) + "\n")
		file.write("[gps]" + "\n")
		file.write("mode: " + self.gps_mode + "\n")
		file.write("timeout: " + str(self.gps_timeout_secs) + "\n")
		file.write("time: " + self.gps_time_flag + "\n")
		file.write("[style]" + "\n")
		style_string = self.graphics_style
		for style in self.graphics_style_all:
			if style != self.graphics_style:
				style_string = style_string + " " + style
		file.write("graphics: " + style_string + "\n")
		file.write("text: " + self.text_flag + "\n")
		file.write("full: " + self.full_window_flag + "\n")
		file.write("clock: " + self.clock_flag + "\n")
		file.write("astro: " + self.astro_flag + "\n")
		file.write("trails: " + self.trails_flag + "\n")
		file.write("skygrid: " + self.sky_grid_flag + "\n")
		file.write("earthgrid: " + self.earth_grid_flag + "\n")
		file.write("moongrid: " + self.moon_grid_flag + "\n")
		file.write("planets: " + self.planet_mode + "\n")
		file.write("planetlabels: " + self.planet_label_flag + "\n")
		file.write("stars: " + self.star_mode + "\n")
		file.write("starlabels: " + self.star_label_flag + "\n")
		file.write("cities: " + self.city_flag + "\n")
		file.write("citylabels: " + self.city_label_flag + "\n")
		file.close()

		return

    	def item_save_activated_test(self, widget):
		config = ConfigParser.ConfigParser()
		#config = ConfigParser.RawConfigParser()
		ini_file = os.path.expanduser("~") + "/.mchronos/mchronos.ini2"
		file = open(ini_file, "w")

		config.add_section('location')
		config.set('location', 'longitude', str(float(self.local.long) * 180/pi))
		config.set('location', 'latitude', str(float(self.local.lat) * 180/pi))

		config.add_section('gps')
		config.set('gps', 'mode', self.gps_mode)
		config.set('gps', 'timeout', str(self.gps_timeout_secs))
		config.set('gps', 'time', self.gps_time_flag)

		config.add_section('style')
		style_string = graphics_style
		for style in graphics_style_all:
			if style != graphics_style:
				style_string = style_string + " " + style
		config.set('style', 'graphics', style_string)
		config.set('style', 'text', self.text_flag)
		config.set('style', 'full', self.full_window_flag)
		config.set('style', 'clock', self.clock_flag)
		config.set('style', 'astro', self.astro_flag)
		config.set('style', 'trails', self.trails_flag)
		config.set('style', 'skygrid', self.sky_grid_flag)
		config.set('style', 'earthgrid', self.earth_grid_flag)
		config.set('style', 'moongrid', self.moon_grid_flag)
		config.set('style', 'planets', self.planet_mode)
		config.set('style', 'planetlabels', self.planet_label_flag)
		config.set('style', 'stars', self.star_mode)
		config.set('style', 'starlabels', star_label_flag)
		config.set('style', 'cities', self.city_flag)
		config.set('style', 'citylabels', self.city_label_flag)

		config.write(file)
		file.close()

		return

        def button_press_event(self, widget, event):
                self.prev_event_x = self.event_x
                self.prev_event_y = self.event_y

                self.event_x = event.x
                self.event_y = event.y

		#toggle_x = 0
		toggle_x = 25

                if (self.prev_event_x != event.x) or (self.prev_event_y != event.y):
			if event.button == 1:
				if event.x >= toggle_x and event.x <= (toggle_x + 20) and event.y >= (self.height-20):
					if self.text_flag == "on":
						self.text_flag = "off"
					else:
						self.text_flag = "on"
					self.resize_window()
					self.click_count = self.click_count + 1

				toggle_x = toggle_x + 25
				if event.x >= toggle_x and event.x <= (toggle_x + 20) and event.y >= (self.height-20):
					if self.full_window_flag == "on":
						self.full_window_flag = "off"
					else:
						self.full_window_flag = "on"
					self.resize_window()
					self.click_count = self.click_count + 10

				toggle_x = toggle_x + 25
				if event.x >= toggle_x and event.x <= (toggle_x + 20) and event.y >= (self.height-20):
					if self.gps_mode in ["on", "lock"]:
						self.gps_mode = "off"
					else:
						self.gps_mode = "lock"
					self.resize_window()

				choice_x = toggle_x + 30

				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 0
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 1
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 2
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 3
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 4
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 5
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 6
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

				choice_x = choice_x + 25
				if event.x >= choice_x and event.x <= (choice_x + 20) and event.y >= (self.height-20):
					self.graphics_style_pos = 7
					self.graphics_style = self.graphics_style_all[self.graphics_style_pos]
					self.resize_window()

                return False

	def expose(self, widget, event):
		if gpsbt_flag == True and (self.gps_mode == "on" or (self.gps_mode == "lock" and self.gps_lock == False)):
			context = gpsbt.start()

			if context == None:
				self.gps_lock = False
				self.local.date = ephem.now()
			else:
				# ensure that GPS device is ready to connect and to receive commands
				time.sleep(2)
				gpsdevice = gpsbt.gps()

				gpsdevice.get_fix()

				gps_timer = 0

				while gpsdevice.get_position()[0] == 0.0:
					time.sleep(1)
					gps_timer = gps_timer + 1
					if gps_timer > int(self.gps_timeout_secs):
						break

				if gpsdevice.fix.latitude == 0.0:
					self.local.lat = self.startup_lat
					self.local.long = self.startup_long
					self.gps_lock = False
					self.local.date = ephem.now()
				else:
					self.local.lat = str(gpsdevice.fix.latitude)
					self.local.long = str(gpsdevice.fix.longitude)
					if self.gps_time_flag == "on":
						gps_seconds = gpsdevice.fix.time
						if gps_seconds != 10000000000.0:
							self.local.date = time.gmtime(gps_seconds)[0:6]
					else:
						self.local.date = ephem.now()
					self.gps_lock = True

				# ends Bluetooth connection
				gpsbt.stop(context)
		else:
			self.local.date = ephem.now()

		self.width, self.height = self.allocation[2], self.allocation[3]

		#Get a cairo context
		self.cr = widget.window.cairo_create()

		if self.supports_alpha == True:
			if self.text_flag == "on" or self.graphics_style in ["bar", "info"]:
				self.cr.set_source_rgba(.0, .0, .0, 0.5) # Transparent black
			else:
				self.cr.set_source_rgba(1.0, 1.0, 1.0, 0.0) # Transparent white
		else:
			self.cr.set_source_rgb(1.0, 1.0, 1.0) # Opaque white

		# Draw the background
		self.cr.set_operator(cairo.OPERATOR_SOURCE)
		if self.graphics_style != "moon":
			self.cr.paint()

		screen_size = self.get_size_request()
		screen_width = screen_size[0]
		screen_height = screen_size[1]
		screen_ratio = 400.0 / screen_height

		if self.graphics_style in ["earth", "horizon"]:
			if self.full_window_flag == "off":
				top_start = self.height * 0.52
			else:
				top_start = self.height * 0.47
		elif self.graphics_style == "info":
			top_start = self.height * 0.30
		else:
			top_start = self.height * 0.03 

		font_size = 11
		self.cr.set_font_size(font_size)
		self.cr.select_font_face("Courier", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)

		if self.text_flag == "on":
			text_height = self.height * 0.013 * screen_ratio
			if self.full_window_flag == "off":
				self.circle_x_center = self.width * 0.5
				if self.graphics_style == "pole":
					self.circle_y_center = self.height * 0.45
				else:
					self.circle_y_center = self.height * 0.3
			else:
				self.circle_x_center = self.width - (self.height / 2.0)
				if self.graphics_style == "pole":
					self.circle_y_center = self.height * 0.5
				else:
					self.circle_y_center = self.height * 0.5
		else:
			text_height = 0.0
			self.circle_x_center = self.width * 0.5
			self.circle_y_center = self.height * 0.5

		if self.graphics_style == "bar":
			bar_height = self.height * 0.07 * screen_ratio
			tic_height = self.height * 0.02 * screen_ratio
			section_gap = self.height * 0.02 * screen_ratio
		else:
			bar_height = 0.0
			tic_height = 0.0
			section_gap = self.height * 0.01 * screen_ratio

		if self.graphics_style in ["sphere", "dome", "pole"]:
			if self.text_flag == "off":
				if self.full_window_flag == "off":
					circle1 = self.width / 2
				else:
					circle1 = self.height / 2
			else:
				if self.graphics_style == "pole":
					if self.full_window_flag == "off":
						circle1 = self.width / 4.0
					else:
						circle1 = self.width / 4.0
				else:
					circle1 = self.width / 4.5
			circle2 = circle1 * 2/3
			circle3 = circle1 * 1/3
			circ_diff = (circle1 - circle2)/2

			sun_day_circle = circle1
			sun_year_circle = circle2
			moon_month_circle = circle3

		if self.graphics_style == "bar":
			size = self.height * 0.035 * screen_ratio
		elif self.graphics_style == "sphere":
			size = circle1 / 6
		elif self.graphics_style == "dome":
			size = circle1 / 10

		if self.graphics_style == "bar":
			bar1_start = top_start + (text_height*6) + section_gap
		elif self.graphics_style == "info":
			bar1_start = top_start + (text_height*4) + section_gap
		elif self.graphics_style in ["earth", "horizon"]:
			if self.full_window_flag == "off":
				bar1_start = self.height * 0.58
			else:
				bar1_start = self.height * 0.54
		else:
			if self.full_window_flag == "off":
				bar1_start = self.height * 0.45
			else:
				bar1_start = top_start + (text_height*5) + section_gap

		if self.graphics_style == "info":
			bar2_start = bar_height + tic_height*2 + (text_height * 20) + section_gap + bar1_start + section_gap
		else:
			bar2_start = bar_height + tic_height*2 + (text_height * 14) + section_gap + bar1_start

		if self.graphics_style == "moon":
			if self.full_window_flag == "off":
				bar3_start = 200
			else:
				bar3_start = 0
		elif self.graphics_style in ["earth", "horizon"]:
			if self.full_window_flag == "off":
				bar3_start = self.height * 0.78
			else:
				bar3_start = self.height * 0.74
		else:
			bar3_start = bar_height + tic_height*2 + (text_height * 10) + section_gap + bar2_start

		save = self.local.date
		
		sun = ephem.Sun()
		sun.compute(self.local)
		
		sun_alt = float(sun.alt) * 180/pi
		sun_az = float(sun.az) * 180/pi

		moon = ephem.Moon()
		moon.compute(self.local)

		moon_alt = float(moon.alt) * 180/pi
		moon_az = float(moon.az) * 180/pi

		# day section

		timezone_offset = ephem.date(self.local.date)-ephem.date(ephem.localtime(self.local.date))
		timezone_hour = round(-timezone_offset * 24)
		start_of_day = ephem.date(str(self.local.date.tuple()[0])+'/'+str(self.local.date.tuple()[1])+'/'+str(self.local.date.tuple()[2])+' 00:00') #reset to start of day GMT
		self.local.date = start_of_day+timezone_offset # compensate for differing time zone

		sun_antitransit = self.local.next_antitransit(sun)
		save2 = ephem.date(self.local.date + 0.001) # this is the best time to set the start of day to since it avoids events from previous day
		
		sun_transit = self.local.next_transit(sun)
		self.local.date = save2
		
		sun_next_antitransit = self.local.next_antitransit(sun)

		self.local.horizon = '-18:00:00'
		try:
			sun_astro_twilight_dawn_txt = "OK"
			sun_astro_twilight_dawn = self.local.next_rising(sun)
		except ephem.AlwaysUpError:
			sun_astro_twilight_dawn_txt = "Always Up"
			sun_astro_twilight_dawn = sun_next_antitransit
		except ephem.NeverUpError:
			sun_astro_twilight_dawn_txt = "Never Up"
			sun_astro_twilight_dawn = sun_transit
		self.local.date = save2
		try:
			sun_astro_twilight_dusk_txt = "OK"
			sun_astro_twilight_dusk = self.local.next_setting(sun)
		except ephem.AlwaysUpError:
			sun_astro_twilight_dusk_txt = "Always Up"
			sun_astro_twilight_dusk = sun_next_antitransit
		except ephem.NeverUpError:
			sun_astro_twilight_dusk_txt = "Never Up"
			sun_astro_twilight_dusk = sun_transit
		self.local.date = save2
	
		self.local.horizon = '-12:00:00'
		try:
			sun_naut_twilight_dawn_txt = "OK"
			sun_naut_twilight_dawn = self.local.next_rising(sun)
		except ephem.AlwaysUpError:
			sun_naut_twilight_dawn_txt = "Always Up"
			sun_naut_twilight_dawn = sun_next_antitransit
		except ephem.NeverUpError:
			sun_naut_twilight_dawn_txt = "Never Up"
			sun_naut_twilight_dawn = sun_transit
		self.local.date = save2
		try:
			sun_naut_twilight_dusk_txt = "OK"
			sun_naut_twilight_dusk = self.local.next_setting(sun)
		except ephem.AlwaysUpError:
			sun_naut_twilight_dusk_txt = "Always Up"
			sun_naut_twilight_dusk = sun_next_antitransit
		except ephem.NeverUpError:
			sun_naut_twilight_dusk_txt = "Never Up"
			sun_naut_twilight_dusk = sun_transit
		self.local.date = save2
		
		self.local.horizon = '-06:00:00'
		try:
			sun_civil_twilight_dawn_txt = "OK"
			sun_civil_twilight_dawn = self.local.next_rising(sun)
		except ephem.AlwaysUpError:
			sun_civil_twilight_dawn_txt = "Always Up"
			sun_civil_twilight_dawn = sun_next_antitransit
		except ephem.NeverUpError:
			sun_civil_twilight_dawn_txt = "Never Up"
			sun_civil_twilight_dawn = sun_transit
		self.local.date = save2
		try:
			sun_civil_twilight_dusk_txt = "OK"
			sun_civil_twilight_dusk = self.local.next_setting(sun)
		except ephem.AlwaysUpError:
			sun_civil_twilight_dusk_txt = "Always Up"
			sun_civil_twilight_dusk = sun_next_antitransit
		except ephem.NeverUpError:
			sun_civil_twilight_dusk_txt = "Never Up"
			sun_civil_twilight_dusk = sun_transit
		self.local.date = save2
	
                sun_always_up_flag = False
                sun_always_down_flag = False

                self.local.horizon = '00:00:00'
                try:
                        sun_rise_txt = "OK"
                        sun_rise = self.local.next_rising(sun)
                except ephem.AlwaysUpError:
                        sun_rise_txt = "Always Up"
                        sun_rise = sun_next_antitransit
                        sun_always_up_flag = True
                except ephem.NeverUpError:
                        sun_rise_txt = "Never Up"
                        sun_rise = sun_transit
                        sun_always_down_flag = True
                self.local.date = save2
		try:
			sun_set_txt = "OK"
			sun_set = self.local.next_setting(sun)
		except ephem.AlwaysUpError:
			sun_set_txt = "Always Up"
			sun_set = sun_next_antitransit
		except ephem.NeverUpError:
			sun_set_txt = "Never Up"
			sun_set = sun_transit
		self.local.date = save2

		moon_antitransit = self.local.next_antitransit(moon)
		save2 = ephem.date(self.local.date + 0.001) # this is the best time to set the start of day to since it avoids events from previous day
		self.local.date = save2
		
		moon_transit = self.local.next_transit(moon)
		self.local.date = save2
		
		moon_next_antitransit = self.local.next_antitransit(moon)
		self.local.date = save2
		
		try:
			moon_rise_txt = "OK"
			moon_rise = self.local.next_rising(moon)
		except ephem.AlwaysUpError:
			moon_rise_txt = "Always Up"
			moon_rise = moon_antitransit
		except ephem.NeverUpError:
			moon_rise_txt = "Never Up"
			moon_rise = moon_transit
		try:
			moon_set_txt = "OK"
			moon_set = self.local.next_setting(moon)
		except ephem.AlwaysUpError:
			moon_set_txt = "Always Up"
			moon_set = moon_antitransit
		except ephem.NeverUpError:
			moon_set_txt = "Never Up"
			moon_set = moon_transit

		self.local.date = save
	
	 	sun_now = self.local.date
 	
		local_sun_antitransit = ephem.Date(ephem.localtime(sun_antitransit))
		local_sun_astro_twilight_dawn = ephem.Date(ephem.localtime(sun_astro_twilight_dawn))
		local_sun_naut_twilight_dawn = ephem.Date(ephem.localtime(sun_naut_twilight_dawn))
		local_sun_civil_twilight_dawn = ephem.Date(ephem.localtime(sun_civil_twilight_dawn))
		local_sun_rise = ephem.Date(ephem.localtime(sun_rise))
		local_sun_transit = ephem.Date(ephem.localtime(sun_transit))
		local_sun_set = ephem.Date(ephem.localtime(sun_set))
		local_sun_civil_twilight_dusk = ephem.Date(ephem.localtime(sun_civil_twilight_dusk))
		local_sun_naut_twilight_dusk = ephem.Date(ephem.localtime(sun_naut_twilight_dusk))
		local_sun_astro_twilight_dusk = ephem.Date(ephem.localtime(sun_astro_twilight_dusk))
		local_sun_now = ephem.Date(ephem.localtime(self.local.date))
	
		local_moon_antitransit = ephem.Date(ephem.localtime(moon_antitransit))
		local_moon_rise = ephem.Date(ephem.localtime(moon_rise))
		local_moon_transit = ephem.Date(ephem.localtime(moon_transit))
		local_moon_set = ephem.Date(ephem.localtime(moon_set))

		local_sun_antitransit_hour = local_sun_antitransit.triple()[2] - int(local_sun_antitransit.triple()[2])
		local_sun_astro_twilight_dawn_hour = local_sun_astro_twilight_dawn.triple()[2] - int(local_sun_astro_twilight_dawn.triple()[2])
		local_sun_naut_twilight_dawn_hour = local_sun_naut_twilight_dawn.triple()[2] - int(local_sun_naut_twilight_dawn.triple()[2])
		local_sun_civil_twilight_dawn_hour = local_sun_civil_twilight_dawn.triple()[2] - int(local_sun_civil_twilight_dawn.triple()[2])
		local_sun_rise_hour = local_sun_rise.triple()[2] - int(local_sun_rise.triple()[2])
		local_sun_transit_hour = local_sun_transit.triple()[2] - int(local_sun_transit.triple()[2])

		local_sun_set_hour = local_sun_set.triple()[2] - int(local_sun_set.triple()[2])
		if int(local_sun_set.triple()[2]) != int(local_sun_transit.triple()[2]):
			local_sun_set_hour = 1.0

		local_sun_civil_twilight_dusk_hour = local_sun_civil_twilight_dusk.triple()[2] - int(local_sun_civil_twilight_dusk.triple()[2])
		if int(local_sun_civil_twilight_dusk.triple()[2]) != int(local_sun_transit.triple()[2]):
			local_sun_civil_twilight_dusk_hour = 1.0

		local_sun_naut_twilight_dusk_hour = local_sun_naut_twilight_dusk.triple()[2] - int(local_sun_naut_twilight_dusk.triple()[2])
		if int(local_sun_naut_twilight_dusk.triple()[2]) != int(local_sun_transit.triple()[2]):
			local_sun_naut_twilight_dusk_hour = 1.0

		local_sun_astro_twilight_dusk_hour = local_sun_astro_twilight_dusk.triple()[2] - int(local_sun_astro_twilight_dusk.triple()[2])
		if int(local_sun_astro_twilight_dusk.triple()[2]) != int(local_sun_transit.triple()[2]):
			local_sun_astro_twilight_dusk_hour = 1.0 
		local_sun_now_hour = local_sun_now.triple()[2] - int(local_sun_now.triple()[2])

                if sun_always_up_flag == True:
                        local_sun_rise_hour = 0.0
                        local_sun_civil_twilight_dawn_hour = 0.0
                        local_sun_naut_twilight_dawn_hour = 0.0
                        local_sun_astro_twilight_dawn_hour = 0.0

		day_length = round((local_sun_set_hour - local_sun_rise_hour) * 24,1)
	
		current_local_date = sun_now
	
		if (local_sun_now_hour < local_sun_astro_twilight_dawn_hour):
			sun_phase = " Night"
		if (local_sun_now_hour >= local_sun_astro_twilight_dawn_hour) and (local_sun_now_hour < local_sun_naut_twilight_dawn_hour):
			#sun_phase = " Astronomical Twilight"
			sun_phase = " Dawn"
		if (local_sun_now_hour >= local_sun_naut_twilight_dawn_hour) and (local_sun_now_hour < local_sun_civil_twilight_dawn_hour):
			#sun_phase = " Nautical Twilight"
			sun_phase = " Dawn"
		if (local_sun_now_hour >= local_sun_civil_twilight_dawn_hour) and (local_sun_now_hour < local_sun_rise_hour):
			#sun_phase = " Civil Twilight"
			sun_phase = " Dawn"
		if (local_sun_now_hour >= local_sun_rise_hour) and (local_sun_now_hour < local_sun_set_hour):
			sun_phase = " Daylight"
		if (local_sun_now_hour >= local_sun_set_hour) and (local_sun_now_hour < local_sun_civil_twilight_dusk_hour):
			#sun_phase = " Civil Twilight"
			sun_phase = " Dusk"
		if (local_sun_now_hour >= local_sun_civil_twilight_dusk_hour) and (local_sun_now_hour < local_sun_naut_twilight_dusk_hour):
			#sun_phase = " Nautical Twilight"
			sun_phase = " Dusk"
		if (local_sun_now_hour >= local_sun_naut_twilight_dusk_hour) and (local_sun_now_hour < local_sun_astro_twilight_dusk_hour):
			#sun_phase = " Astronomical Twilight"
			sun_phase = " Dusk"
		if (local_sun_now_hour >= local_sun_astro_twilight_dusk_hour):
			sun_phase = " Night"

		if ((ephem.date(self.local.date)-ephem.date(ephem.localtime(self.local.date)))*24) == (time.timezone/60/60):
			local_time_zone_daylight_flag = 0
		else:
			local_time_zone_daylight_flag = time.daylight

		local_time_zone_name = time.tzname
		zone = local_time_zone_name[local_time_zone_daylight_flag]

		week_days = ["Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
		day_of_week = week_days[time.localtime(time.time())[6]]
	
		# pole circle
		if self.graphics_style == "pole":
			radius = sun_day_circle
	
			self.cr.set_source_rgb(0, 0, 0)

			self.cr.move_to(self.circle_x_center, self.circle_y_center)
			self.cr.arc(self.circle_x_center, self.circle_y_center, radius, 0, 2.0*3.14)
			self.cr.fill()
			self.cr.stroke()

			self.cr.set_source_rgb(1.0, 1.0, 1.0)
			self.cr.move_to(self.circle_x_center - 5, self.circle_y_center - radius + 10)
			self.cr.text_path("N")
			self.cr.fill()
			self.cr.stroke()
			self.cr.move_to(self.circle_x_center + radius - 10, self.circle_y_center + 5)
			if self.astro_sign == 1:
				self.cr.text_path("E")
			else:
				self.cr.text_path("W")
			self.cr.fill()
			self.cr.stroke()
			self.cr.move_to(self.circle_x_center - 5, self.circle_y_center + radius - 2)
			self.cr.text_path("S")
			self.cr.fill()
			self.cr.stroke()
			self.cr.move_to(self.circle_x_center - radius, self.circle_y_center + 5)
			if self.astro_sign == 1:
				self.cr.text_path("W")
			else:
				self.cr.text_path("E")
			self.cr.fill()
			self.cr.stroke()

			self.cr.set_source_rgb(1.0, 0.0, 0.0)
			self.cr.set_line_width(1.0)
			for alt in range (890, 895, 1):
				sub_alt = alt/10.0
				radius = sun_day_circle * ((90.0 - sub_alt) / 1.0)
				self.cr.arc(self.circle_x_center, self.circle_y_center, radius, 0, 2.0*3.14)
				self.cr.stroke()
				
			color = [1.00, 0.00, 0.00]
			radius = sun_day_circle
			for az in range (0, 360, 15):
				pos = az / 360.0
				self.radial_line(pos, radius, color)

			size = radius * 0.60
			radius = 0.0
			color = [0, 0, 0]
			self.radial_ball(pos, radius, size, color, True)

			color = [1.00, 0.00, 0.00]
			radius = sun_day_circle
			for az in range (0, 360, 90):
				pos = az / 360.0
				self.radial_line(pos, radius, color)

			lat_save = self.local.lat
			self.local.lat = "90"
			star = ephem.star("Polaris")
			star.compute(self.local)
			star_alt = float(star.alt) * 180/pi
			star_az = self.astro_sign * float(star.az) * 180/pi
			pos = (star_az - 180.0) / 360.0
			color = [1.0, 1.0, 1.0]
			radius2 = sun_day_circle * ((90.0 - star_alt) / 1.0)
			dot = 5
			self.radial_ball(pos, radius2, dot, color, True)
			self.radial_text(pos, radius2 * 0.8, color, "Polaris")
			self.radial_text(0.5, radius2 * 0.1, color, "Celestial")
			self.radial_text(0.0, radius2 * 0.2, color, "Pole")
			self.local.lat = lat_save
			star.compute(self.local)

		# earth map
		if self.graphics_style == "earth":
			if self.full_window_flag == "off":
				image = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_earth_400x200.png")
				image2 = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_earth_400x200.png")
			else:
				image = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_earth_720x360.png")
				image2 = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_earth_720x360.png")

			image_width = image.get_width()
			image_height = image.get_height()

			self.cr.set_source_surface(image, 0, 0)
			self.cr.paint()

	                size = self.width / 50

			# Calculate sub-solar point
			sun_date_save = self.local.date
			long_save = self.local.long

			# temporarily set longitude to 0 so as to calculate sun transit even without self.local longitude being set correctly
			self.local.long = "0"
			start_of_day = ephem.date(str(self.local.date.tuple()[0])+'/'+str(self.local.date.tuple()[1])+'/'+str(self.local.date.tuple()[2])+' 00:00') #reset to start of day GMT
			self.local.date = start_of_day
			sun = ephem.Sun()
			sun.compute(self.local)
			sun_next_transit = self.local.next_transit(sun)
			self.local.date = sun_date_save
			sun_long = -(((self.local.date - sun_next_transit) * 24) - (self.local.long * 180/pi)/15) * 15
			sun_lat = -sun.dec * 180/pi
			self.local.long = long_save

			# temporarily set longitude to 0 so as to calculate moon transit even without self.local longitude being set correctly
			self.local.long = "0"
			start_of_day = ephem.date(str(self.local.date.tuple()[0])+'/'+str(self.local.date.tuple()[1])+'/'+str(self.local.date.tuple()[2])+' 00:00') #reset to start of day GMT
			self.local.date = start_of_day
			moon = ephem.Moon()
			moon.compute(self.local)
			moon_next_transit = self.local.next_transit(moon)
			moon_next_transit2 = self.local.next_transit(moon)
			moon_speed = 360 / ((moon_next_transit2 - moon_next_transit) * 24.0)
			self.local.date = sun_date_save
			moon_long = -(((self.local.date - moon_next_transit) * 24) - (self.local.long * 180/pi)/15) * moon_speed
			moon_lat = -moon.dec * 180/pi
			self.local.long = long_save

			if moon_long < -180:
				moon_long = moon_long + 360
			if moon_long > 180:
				moon_long = moon_long - 360

                        prev_x = -999
                        prev_y = -999
                        prev_dest_lon = -999
                        d = pi/2
                        for tc in range(0,365,5):
                                dest_lat = asin(sin(sun_lat*pi/180)*cos(d)+cos(sun_lat*pi/180)*sin(d)*cos(tc*pi/180))
                                temp_lon = atan2(sin(tc*pi/180)*sin(d)*cos(sun_lat*pi/180),cos(d)-sin(sun_lat*pi/180)*sin(dest_lat))
                                dest_lon = ((sun_long*pi/180)+temp_lon+pi) % (2*pi) - pi

                                dest_lat = dest_lat * 180/pi
                                dest_lon = dest_lon * 180/pi

                                x = (dest_lon/180.0) * (image_width/2.0) + (image_width/2.0)
                                y = ((dest_lat/90.0) * (image_height/2.0) + (image_height/2.0)) 

                                if (prev_dest_lon - dest_lon) > 180 and (prev_dest_lon != -999):
                                        self.cr.line_to(image_width-1,y)
                                        self.cr.line_to(image_width-1,image_height-1)
                                        self.cr.line_to(1,image_height-1)
                                        self.cr.line_to(1,y)
                                        prev_x = x
                                        prev_y = y
                                elif (dest_lon - prev_dest_lon) > 180 and (prev_dest_lon != -999):
                                        self.cr.line_to(1,y)
                                        self.cr.line_to(1,1)
                                        self.cr.line_to(image_width-1,1)
                                        self.cr.line_to(image_width-1,y)
                                        prev_x = x
                                        prev_y = y
                                if prev_x != -999:
                                        self.cr.line_to(x,y)
                                prev_x = x
                                prev_y = y
                                prev_dest_lon = dest_lon

			self.cr.set_source_rgb(0,0,0)
                        self.cr.paint_with_alpha(0.1)
                        #self.cr.paint()
                        self.cr.fill()
                        self.cr.stroke()
			self.cr.set_source_surface(image2, 0, 0)
			self.cr.paint_with_alpha(0.3)

			if self.earth_grid_flag == "on":
				self.cr.set_line_width(0.3)
				self.cr.set_source_rgb(1.0, 1.0, 1.0)
				for long in range (-180, 180, 15):
					lat = 90
					x = (long/180.0) * (image_width/2.0) + (image_width/2.0)
					y = ((lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
					self.cr.move_to(x, y)
					lat = -90
					x = (long/180.0) * (image_width/2.0) + (image_width/2.0)
					y = ((lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
					self.cr.line_to(x, y)
					self.cr.stroke()
				for lat in range (-90, 90, 15):
					long = -180
					x = (long/180.0) * (image_width/2.0) + (image_width/2.0)
					y = ((lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
					self.cr.move_to(x, y)
					long = 180
					x = (long/180.0) * (image_width/2.0) + (image_width/2.0)
					y = ((lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
					self.cr.line_to(x, y)
					self.cr.stroke()

                        # plot sub-solar image
                        x = (sun_long/180.0) * (image_width/2.0) + (image_width/2.0)
                        y = ((sun_lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
                        self.cr.set_source_rgb(1.0, 1.0, 0.0)
                        self.cr.arc(x, y, size, 0, 2 * math.pi)
                        self.cr.fill()
                        self.cr.stroke()
			
                        # plot sub-lunar image
                        #x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0)
                        #y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
                        #self.cr.set_source_rgb(0.5, 0.5, 0.5)
                        #self.cr.arc(x, y, size, 0, 2 * math.pi)
                        #self.cr.fill()
                        #self.cr.stroke()

			if (self.event_x <= image_width) and (self.event_y <= image_height):		
				if self.reset_flag == "off":
					font_size = 25
					self.cr.set_font_size(font_size)

					long = int(round((self.event_x - (image_width / 2.0)) / (image_width / 2.0) * 180, 0))
					lat = -int(round((self.event_y - (image_height / 2.0)) / (image_height / 2.0) * 90, 0))

					self.cr.move_to(image_width * 0.5, image_height * 0.65)
					if lat >= 0:
						lat_dir = "N"
					else:
						lat_dir = "S"
					self.cr.fill()
					self.cr.stroke()

					self.cr.move_to(image_width * 0.5, image_height * 0.65)
					if long >= 0:
						long_dir = "E"
					else:
						long_dir = "W"
					self.cr.text_path(str(lat).zfill(2) + u"\xb0" + lat_dir + " " + str(long).zfill(3) + u"\xb0" + long_dir)
					self.cr.fill()
					self.cr.stroke()

					font_size = 11
					self.cr.set_font_size(font_size)

					self.local.lat = str(lat)
					self.local.long = str(long)
					self.reset_flag = "on"
				else:
					self.reset_flag = "off"
					self.local.lat = self.startup_lat
					self.local.long = self.startup_long

                        # plot observer location
                        latitude = -self.local.lat * 180/pi
                        longitude = self.local.long * 180/pi
                        x = (longitude/180.0) * (image_width/2.0) + (image_width/2.0)
                        y = ((latitude/90.0) * (image_height/2.0) + (image_height/2.0)) 
			# plot gps status indicator
			if self.gps_mode == "on" or self.gps_mode == "lock":
				if self.gps_mode == "on":
					if self.gps_lock == True: 
						self.cr.set_source_rgb(0.00, 1.00, 0.00)
					else:
						self.cr.set_source_rgb(1.00, 0.00, 0.00)

				if self.gps_mode == "lock":
					if self.gps_lock == True: 
						self.cr.set_source_rgb(1.00, 0.65, 0.00)
					else:
						self.cr.set_source_rgb(1.00, 0.00, 0.00)
			else:
				self.cr.set_source_rgb(1.00, 1.00, 1.00)
                        self.cr.arc(x, y, size/2, 0, 2 * math.pi)
                        self.cr.fill()
                        self.cr.stroke()

			# plot city locations
			if self.city_flag == "on":
				self.cr.set_line_width(1.0)
				for city in cities._city_data.items():
					city_name = city[0]
					city_lat = float(city[1][0])
					city_long = float(city[1][1])
					latitude = -city_lat
					longitude = city_long
					x = (longitude/180.0) * (image_width/2.0) + (image_width/2.0)
					y = ((latitude/90.0) * (image_height/2.0) + (image_height/2.0)) 
					self.cr.set_source_rgb(1.00, 1.00, 1.00)
					self.cr.arc(x, y, size/4, 0, 2 * math.pi)
					#self.cr.fill()
					self.cr.stroke()
					if self.city_label_flag == "on":
						self.cr.move_to(x + 5, y + 5)
						self.cr.text_path(city_name)
						self.cr.fill()
						self.cr.stroke()

			if self.clock_flag == "on":	
				#if (self.gps_lock == False) or (self.gps_time_flag == "off"):
				#	self.cr.set_source_rgb(1.0, 1.0, 1.0)
				#else:
				#	self.cr.set_source_rgb(0.0, 1.0, 0.0)

				#self.cr.set_source_rgb(1.0, 1.0, 1.0) # inherit color from gps status above instead
				font_size = 25
				self.cr.set_font_size(font_size)

				self.cr.move_to(image_width * 0.6, image_height * 0.85)
				self.cr.text_path(str(local_sun_now)[:-9])
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(image_width * 0.6, image_height * 0.95)
				self.cr.text_path(str(local_sun_now.tuple()[3]).zfill(2)+":"+str(local_sun_now.tuple()[4]).zfill(2) + " " + zone)
				self.cr.fill()
				self.cr.stroke()

				font_size = 11
				self.cr.set_font_size(font_size)

		# sky horizon
		if self.graphics_style == "horizon":
			if sun_alt > 0:	
				self.cr.set_source_rgb(0.015625, 0.515625, 0.984375)
				if self.star_mode == "auto":
					self.star_flag = "off"
				if self.planet_mode == "auto":
					self.planet_flag = "off"
			elif sun_alt < -18:	
				self.cr.set_source_rgb(0, 0, 0)
				if self.star_mode == "auto":
					self.star_flag = "on"
				if self.planet_mode == "auto":
					self.planet_flag = "on"
			else:
				dim_factor = 1.0 - (abs(sun_alt) / 18.0)
				self.cr.set_source_rgb(0.015625 * dim_factor, 0.515625 * dim_factor, 0.984375 * dim_factor)
				if self.star_mode == "auto":
					self.star_flag = "on"
				if self.planet_mode == "auto":
					self.planet_flag = "on"

			self.cr.rectangle(0, 0, self.width, self.width/4.0)
			self.cr.fill()
			self.cr.stroke()

			self.cr.set_source_rgb(0.0, 0.0, 0.0)
			self.cr.rectangle(0, self.width/4.0, self.width, self.width/4.0)
			self.cr.fill()
			self.cr.stroke()

			if self.sky_grid_flag == "on":
				self.cr.set_source_rgb(1.0, 1.0, 1.0)
				self.cr.set_line_width(0.3)

				for alt in range (-90, 90, 10):
					self.cr.move_to(0, ((90 - alt) / 90.0) * (self.width / 4.0))
					self.cr.line_to(self.width, ((90 - alt) / 90.0) * (self.width / 4.0))
					self.cr.stroke()
					
				for az in range (0, 360, 10):
					self.cr.move_to((az / 360.0) * self.width, 0)
					self.cr.line_to((az / 360.0) * self.width, (self.width / 2.0))
					self.cr.stroke()

				self.cr.set_line_width(1.0)

				self.cr.move_to((self.width * 0.25) - 5, 10)
				self.cr.text_path("E")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to((self.width * 0.5) - 5, 10)
				self.cr.text_path("S")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to((self.width * 0.75) - 5, 10)
				self.cr.text_path("W")
				self.cr.fill()
				self.cr.stroke()

			sun.compute(self.local)
			sun_alt = float(sun.alt) * 180/pi
			sun_az = float(sun.az) * 180/pi
			#if self.local.lat < 0:
			#	sun_az = sun_az + 180.0

			moon.compute(self.local)
			moon_alt = float(moon.alt) * 180/pi
			moon_az = float(moon.az) * 180/pi
			#if self.local.lat < 0:
			#	moon_az = moon_az + 180.0
		
			size = self.width / 40	
			self.cr.arc((sun_az / 360.0) * self.width, ((90 - sun_alt) / 90.0) * (self.width / 4.0), size, 0, 2.0*3.14)
			self.cr.set_source_rgb(1.00, 1.00, 0.00)
			self.cr.fill()
			self.cr.stroke()

			if sun_alt > 0:	
				if self.trails_flag == "on":
					# plot sun and moon trails
					size = self.width / 40	
					self.local.date = self.local.date - 0.5
					for hour in range (0, 24, 1):
						self.local.date = self.local.date + (1.0 / 24.0)
						sun.compute(self.local)  
						sun_alt = float(sun.alt) * 180/pi
						if sun_alt >= 0.0:
							sun_az = float(sun.az) * 180/pi
							self.cr.set_source_rgb(1.00, 1.00, 0.00)
							self.cr.arc((sun_az / 360.0) * self.width, ((90 - sun_alt) / 90.0) * (self.width / 4.0), size/4, 0, 2.0*3.14)
							self.cr.fill()
							self.cr.stroke()

						#moon.compute(self.local)  
						#moon_alt = float(moon.alt) * 180/pi
						#if moon_alt >= 0.0:
						#	moon_az = float(moon.az) * 180/pi
						#	self.cr.set_source_rgb(0.7, 0.7, 0.7)
						#	self.cr.arc((moon_az / 360.0) * self.width, ((90 - moon_alt) / 90.0) * (self.width / 4.0), size/4, 0, 2.0*3.14)
						#	self.cr.fill()
						#	self.cr.stroke()

					self.local.date = save

			#if moon_alt >= 0:	
			#	self.cr.arc((moon_az / 360.0) * self.width, ((90 -moon_alt) / 90.0) * (self.width / 4.0), size, 0, 2.0*3.14)
			#	self.cr.set_source_rgb(0.3, 0.3, 0.3)
			#	self.cr.fill()
			#	self.cr.stroke()

			if self.planet_flag == "on":
				mercury = ephem.Mercury()
				mercury.compute(self.local)
				mercury_alt = float(mercury.alt) * 180/pi
				mercury_az = float(mercury.az) * 180/pi
				self.cr.set_source_rgb(0.4, 0.4, 0.4)
				self.cr.arc((mercury_az / 360.0) * self.width, ((90 - mercury_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((mercury_az / 360.0) * self.width + 5, ((90 - mercury_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Mercury")
					self.cr.fill()
					self.cr.stroke()

				venus = ephem.Venus()
				venus.compute(self.local)
				venus_alt = float(venus.alt) * 180/pi
				venus_az = float(venus.az) * 180/pi
				self.cr.set_source_rgb(0.6, 0.6, 0.6)
				self.cr.arc((venus_az / 360.0) * self.width, ((90 - venus_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((venus_az / 360.0) * self.width + 5, ((90 - venus_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Venus")
					self.cr.fill()
					self.cr.stroke()

				mars = ephem.Mars()
				mars.compute(self.local)
				mars_alt = float(mars.alt) * 180/pi
				mars_az = float(mars.az) * 180/pi
				self.cr.set_source_rgb(1.0, 0.0, 0.0)
				self.cr.arc((mars_az / 360.0) * self.width, ((90 - mars_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((mars_az / 360.0) * self.width + 5, ((90 - mars_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Mars")
					self.cr.fill()
					self.cr.stroke()

				jupiter = ephem.Jupiter()
				jupiter.compute(self.local)
				jupiter_alt = float(jupiter.alt) * 180/pi
				jupiter_az = float(jupiter.az) * 180/pi
				self.cr.set_source_rgb(1.0, 0.6, 0.0)
				self.cr.arc((jupiter_az / 360.0) * self.width, ((90 - jupiter_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((jupiter_az / 360.0) * self.width + 5, ((90 - jupiter_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Jupiter")
					self.cr.fill()
					self.cr.stroke()

				saturn = ephem.Saturn()
				saturn.compute(self.local)
				saturn_alt = float(saturn.alt) * 180/pi
				saturn_az = float(saturn.az) * 180/pi
				self.cr.set_source_rgb(0.8, 0.8, 0.0)
				self.cr.arc((saturn_az / 360.0) * self.width, ((90 - saturn_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((saturn_az / 360.0) * self.width + 5, ((90 - saturn_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Saturn")
					self.cr.fill()
					self.cr.stroke()

				uranus = ephem.Uranus()
				uranus.compute(self.local)
				uranus_alt = float(uranus.alt) * 180/pi
				uranus_az = float(uranus.az) * 180/pi
				self.cr.set_source_rgb(0.0, 0.6, 0.6)
				self.cr.arc((uranus_az / 360.0) * self.width, ((90 - uranus_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((uranus_az / 360.0) * self.width + 5, ((90 - uranus_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Uranus")
					self.cr.fill()
					self.cr.stroke()

				neptune = ephem.Neptune()
				neptune.compute(self.local)
				neptune_alt = float(neptune.alt) * 180/pi
				neptune_az = float(neptune.az) * 180/pi
				self.cr.set_source_rgb(0.0, 0.6, 0.6)
				self.cr.arc((neptune_az / 360.0) * self.width, ((90 - neptune_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((neptune_az / 360.0) * self.width + 5, ((90 - neptune_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Neptune")
					self.cr.fill()
					self.cr.stroke()

				pluto = ephem.Pluto()
				pluto.compute(self.local)
				pluto_alt = float(pluto.alt) * 180/pi
				pluto_az = float(pluto.az) * 180/pi
				self.cr.set_source_rgb(0.5, 0.5, 0.5)
				self.cr.arc((pluto_az / 360.0) * self.width, ((90 - pluto_alt) / 90.0) * (self.width / 4.0), size/3, 0, 2.0*3.14)
				self.cr.fill()
				self.cr.stroke()
				if self.planet_label_flag == "on":
					self.cr.set_source_rgb(1.0, 1.0, 1.0)
					self.cr.move_to((pluto_az / 360.0) * self.width + 5, ((90 - pluto_alt) / 90.0) * (self.width / 4.0) + 5)
					self.cr.text_path("Pluto")
					self.cr.fill()
					self.cr.stroke()

			if self.star_flag == "on":
				for body in stars.stars:
					star = ephem.star(body)
					star.compute(self.local)
					star_alt = float(star.alt) * 180/pi
					star_az = float(star.az) * 180/pi
					#if self.local.lat < 0:
					#	star_az = star_az + 180.0
					self.cr.set_source_rgb(1.00, 1.00, 1.00)
					#dot = (self.width / 100.0) - float(star.mag)
					dot = 4 - float(star.mag)
					self.cr.arc((star_az / 360.0) * self.width, ((90 - star_alt) / 90.0) * (self.width / 4.0), dot, 0, 2.0*3.14)
					self.cr.fill()
					self.cr.stroke()
					if self.star_label_flag == "on":
						self.cr.set_source_rgb(1.0, 1.0, 1.0)
						self.cr.move_to((star_az / 360.0) * self.width + 5, ((90 - star_alt) / 90.0) * (self.width / 4.0) + 5)
						self.cr.text_path(star.name)
						self.cr.fill()
						self.cr.stroke()

		# sky dome
		if self.graphics_style == "dome":
			radius = sun_day_circle
			radius2 = sun_day_circle * ((90 - sun_alt) / 90.0)
	
			if sun_alt > 0:	
				self.cr.set_source_rgb(0.015625, 0.515625, 0.984375)
				if self.star_mode == "auto":
					self.star_flag = "off"
				if self.planet_mode == "auto":
					self.planet_flag = "off"
			elif sun_alt < -18:	
				self.cr.set_source_rgb(0, 0, 0)
				if self.star_mode == "auto":
					self.star_flag = "on"
				if self.planet_mode == "auto":
					self.planet_flag = "on"
			else:
				dim_factor = 1.0 - (abs(sun_alt) / 18.0)
				self.cr.set_source_rgb(0.015625 * dim_factor, 0.515625 * dim_factor, 0.984375 * dim_factor)
				if self.star_mode == "auto":
					self.star_flag = "on"
				if self.planet_mode == "auto":
					self.planet_flag = "on"

			self.cr.move_to(self.circle_x_center, self.circle_y_center)
			self.cr.arc(self.circle_x_center, self.circle_y_center, radius, 0, 2.0*3.14)
			self.cr.fill()
			self.cr.stroke()

			if self.sky_grid_flag == "on":
				self.cr.set_source_rgb(1.0, 1.0, 1.0)
				self.cr.move_to(self.circle_x_center - 5, self.circle_y_center - radius + 10)
				self.cr.text_path("N")
				self.cr.fill()
				self.cr.stroke()
				self.cr.move_to(self.circle_x_center + radius - 10, self.circle_y_center + 5)
				if self.astro_sign == 1:
					self.cr.text_path("E")
				else:
					self.cr.text_path("W")
				self.cr.fill()
				self.cr.stroke()
				self.cr.move_to(self.circle_x_center - 5, self.circle_y_center + radius - 2)
				self.cr.text_path("S")
				self.cr.fill()
				self.cr.stroke()
				self.cr.move_to(self.circle_x_center - radius, self.circle_y_center)
				if self.astro_sign == 1:
					self.cr.text_path("W")
				else:
					self.cr.text_path("E")
				self.cr.fill()
				self.cr.stroke()

				self.cr.set_source_rgb(1.0, 1.0, 1.0)
				self.cr.set_line_width(0.3)

				for alt in range (0, 90, 10):
					radius = sun_day_circle * ((90 - alt) / 90.0)
					self.cr.arc(self.circle_x_center, self.circle_y_center, radius, 0, 2.0*3.14)
					self.cr.stroke()
					
				color = [1.00, 1.00, 1.00]
				for az in range (0, 360, 10):
					radius = sun_day_circle
					pos = az / 360.0
					self.radial_line(pos, radius, color)

			self.cr.set_source_rgb(1.0, 1.0, 0.0)

			sun.compute(self.local)
			sun_alt = float(sun.alt) * 180/pi
			sun_az = self.astro_sign * float(sun.az) * 180/pi

			moon.compute(self.local)
			moon_alt = float(moon.alt) * 180/pi
			moon_az = self.astro_sign * float(moon.az) * 180/pi
				
			radius = sun_day_circle
			radius2 = sun_day_circle * ((90 - sun_alt) / 90.0)

			if sun_alt > 0:	
				pos = (sun_az - 180.0) / 360.0
				color = [1.00, 1.00, 0.00]
				self.radial_ball(pos, radius2, size, color, True)
				color = [1.00, 0.50, 0.50]
				self.radial_ball(pos, radius2, size, color, False)

				if self.trails_flag == "on":
					# plot sun and moon trails
					self.local.date = self.local.date - 0.5
					for hour in range (0, 24, 1):
						self.local.date = self.local.date + (1.0 / 24.0)
						sun.compute(self.local)  
						sun_alt = float(sun.alt) * 180/pi
						if sun_alt >= 0.0:
							sun_az = float(sun.az) * 180/pi
							radius2 = sun_day_circle * ((90 - sun_alt) / 90.0)
							pos = (sun_az - 180.0) / 360.0
							color = [1.00, 1.00, 0.00]
							self.radial_ball(pos, radius2, 2, color, True)

					#	moon.compute(self.local)  
					#	moon_alt = float(moon.alt) * 180/pi
					#	if moon_alt >= 0.0:
					#		moon_az = float(moon.az) * 180/pi
					#		radius2 = sun_day_circle * ((90 - moon_alt) / 90.0)
					#		pos = (moon_az - 180.0) / 360.0
					#		color = [0.70, 0.70, 0.70]
					#		self.radial_ball(pos, radius2, 2, color, True)

					self.local.date = save

			#radius = sun_day_circle
			#radius2 = sun_day_circle * ((90 - moon_alt) / 90.0)
			#pos = (moon_az - 180.0) / 360.0
			#color = [0.5, 0.5, 0.5]
			#self.radial_ball(pos, radius2, size, color, True)
			#color = [1.00, 0.50, 0.50]
			#self.radial_ball(pos, radius2, size, color, False)

			if self.planet_flag == "on":
				mercury = ephem.Mercury()
				mercury.compute(self.local)
				mercury_alt = float(mercury.alt) * 180/pi
				if mercury_alt >= 0.0:
					mercury_az = self.astro_sign * float(mercury.az) * 180/pi
					pos = (mercury_az - 180.0) / 360.0
					color = [0.4,0.4,0.4]
					radius2 = sun_day_circle * ((90 - mercury_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Mercury")

				venus = ephem.Venus()
				venus.compute(self.local)
				venus_alt = float(venus.alt) * 180/pi
				if venus_alt >= 0.0:
					venus_az = self.astro_sign * float(venus.az) * 180/pi
					pos = (venus_az - 180.0) / 360.0
					color = [0.6,0.6,0]
					radius2 = sun_day_circle * ((90 - venus_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Venus")

				mars = ephem.Mars()
				mars.compute(self.local)
				mars_alt = float(mars.alt) * 180/pi
				if mars_alt >= 0.0:
					mars_az = self.astro_sign * float(mars.az) * 180/pi
					pos = (mars_az - 180.0) / 360.0
					color = [1,0,0]
					radius2 = sun_day_circle * ((90 - mars_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Mars")

				jupiter = ephem.Jupiter()
				jupiter.compute(self.local)
				jupiter_alt = float(jupiter.alt) * 180/pi
				if jupiter_alt >= 0.0:
					jupiter_az = self.astro_sign * float(jupiter.az) * 180/pi
					pos = (jupiter_az - 180.0) / 360.0
					color = [1,0.6,0]
					radius2 = sun_day_circle * ((90 - jupiter_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Jupiter")

				saturn = ephem.Saturn()
				saturn.compute(self.local)
				saturn_alt = float(saturn.alt) * 180/pi
				if saturn_alt >= 0.0:
					saturn_az = self.astro_sign * float(saturn.az) * 180/pi
					pos = (saturn_az - 180.0) / 360.0
					color = [0.8,0.8,0]
					radius2 = sun_day_circle * ((90 - saturn_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Saturn")

				uranus = ephem.Uranus()
				uranus.compute(self.local)
				uranus_alt = float(uranus.alt) * 180/pi
				if uranus_alt >= 0.0:
					uranus_az = self.astro_sign * float(uranus.az) * 180/pi
					pos = (uranus_az - 180.0) / 360.0
					color = [0,0.6,0.6]
					radius2 = sun_day_circle * ((90 - uranus_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Uranus")

				neptune = ephem.Neptune()
				neptune.compute(self.local)
				neptune_alt = float(neptune.alt) * 180/pi
				if neptune_alt >= 0.0:
					neptune_az = self.astro_sign * float(neptune.az) * 180/pi
					pos = (neptune_az - 180.0) / 360.0
					color = [0,0.6,0.6]
					radius2 = sun_day_circle * ((90 - neptune_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Neptune")

				pluto = ephem.Pluto()
				pluto.compute(self.local)
				pluto_alt = float(pluto.alt) * 180/pi
				if pluto_alt >= 0.0:
					pluto_az = self.astro_sign * float(pluto.az) * 180/pi
					pos = (pluto_az - 180.0) / 360.0
					color = [0.5,0.5,0.5]
					radius2 = sun_day_circle * ((90 - pluto_alt) / 90.0)
					self.radial_ball(pos, radius2, size/3, color, True)
					if self.planet_label_flag == "on":
						color = [1.0, 1.0, 1.0]
						self.radial_text(pos, radius2, color, "Pluto")

			if self.star_flag == "on":
				for body in stars.stars:
					star = ephem.star(body)
					star.compute(self.local)
					star_alt = float(star.alt) * 180/pi
					if star_alt >= 0.0:
						star_az = self.astro_sign * float(star.az) * 180/pi
						pos = (star_az - 180.0) / 360.0
						color = [1.0,1.0,1.0]
						radius2 = sun_day_circle * ((90 - star_alt) / 90.0)
						#dot = 4 - float(star.mag)
						dot = circle1 / 40 - float(star.mag)
						self.radial_ball(pos, radius2, dot, color, True)
						if self.star_label_flag == "on":
							color = [1.0, 1.0, 1.0]
							self.radial_text(pos, radius2, color, star.name)

		# day circle
		if self.graphics_style == "sphere":
			# night slice

			radius = sun_day_circle
			radius2 = sun_day_circle - circ_diff

			red_inc = 0
			green_inc = 0
			blue_inc = 0

			for mult_pos in range(int(local_sun_astro_twilight_dawn_hour*10000), int(local_sun_rise_hour*10000), int((local_sun_rise_hour-local_sun_astro_twilight_dawn_hour)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (local_sun_rise_hour-local_sun_astro_twilight_dawn_hour)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc + 0.0015625
				green_inc = green_inc + 0.0515625
				blue_inc = blue_inc + 0.0984375
				self.pie_slice(start_pos, stop_pos, radius, color)

			start_pos = 0
			stop_pos = local_sun_astro_twilight_dawn_hour
			color = [0,0,0]
			self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 0.015625
			green_inc = 0.515625
			blue_inc = 0.984375

			for mult_pos in range(int(local_sun_rise_hour*10000), int(local_sun_transit_hour*10000), int((local_sun_transit_hour-local_sun_rise_hour)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (local_sun_transit_hour-local_sun_rise_hour)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc + 0.0984375
				green_inc = green_inc + 0.0484375
				blue_inc = blue_inc + 0.0015625
				self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 1
			green_inc = 1 
			blue_inc = 1

			for mult_pos in range(int(local_sun_transit_hour*10000), int(local_sun_set_hour*10000), int((local_sun_set_hour-local_sun_transit_hour)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (local_sun_set_hour-local_sun_transit_hour)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc - 0.0984375
				green_inc = green_inc - 0.0484375
				blue_inc = blue_inc - 0.0015625
				self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 0.015625
			green_inc = 0.515625
			blue_inc = 0.984375

			for mult_pos in range(int(local_sun_set_hour*10000), int(local_sun_astro_twilight_dusk_hour*10000), int((local_sun_astro_twilight_dusk_hour-local_sun_set_hour)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (local_sun_astro_twilight_dusk_hour-local_sun_set_hour)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc - 0.0015625
				green_inc = green_inc - 0.0515625
				blue_inc = blue_inc - 0.0984375
				self.pie_slice(start_pos, stop_pos, radius, color)

			start_pos = local_sun_astro_twilight_dusk_hour
			stop_pos = 1
			color = [0, 0, 0]
			self.pie_slice(start_pos, stop_pos, radius, color)

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.set_line_width(1.0)

			pos = local_sun_antitransit_hour
			color = [1.00, 1.00, 1.00]
			self.radial_line(pos, radius, color)

			pos = local_sun_astro_twilight_dawn_hour
			self.radial_line(pos, radius, color)

			pos = local_sun_naut_twilight_dawn_hour
			self.radial_line(pos, radius, color)

			pos = local_sun_civil_twilight_dawn_hour
			self.radial_line(pos, radius, color)

			pos = local_sun_rise_hour
			self.radial_line(pos, radius, color)

			pos = local_sun_transit_hour
			color = [0.00, 0.00, 0.00]
			self.radial_line(pos, radius, color)

			pos = local_sun_set_hour
			color = [1.00, 1.00, 1.00]
			self.radial_line(pos, radius, color)

			pos = local_sun_civil_twilight_dusk_hour
			self.radial_line(pos, radius, color)

			pos = local_sun_naut_twilight_dusk_hour
			self.radial_line(pos, radius, color)

			pos = local_sun_astro_twilight_dusk_hour
			self.radial_line(pos, radius, color)

			#pos = local_sun_now_hour
			#self.radial_line(pos, radius, color)

			pos = local_sun_now_hour
			color = [1.00, 1.00, 0.00]
			self.radial_ball(pos, radius2, size, color, True)

			if self.gps_mode == "on" or self.gps_mode == "lock":
				if self.gps_mode == "on":
					if self.gps_lock == True: 
						color = [0.00, 1.00, 0.00]
					else:
						color = [1.00, 0.00, 0.00]

				if self.gps_mode == "lock":
					if self.gps_lock == True: 
						color = [1.00, 0.65, 0.00]
					else:
						color = [1.00, 0.00, 0.00]

				self.radial_ball(pos, radius2, size/2, color, True)

			pos = local_sun_now_hour
			color = [1.00, 0.50, 0.50]
			self.radial_ball(pos, radius2, size, color, False)

		# day bars
		if self.graphics_style == "bar":
			self.cr.set_source_rgb(0.00, 0.00, 0.00)

			self.cr.rectangle(self.width*0.05,bar1_start,self.width*0.90*local_sun_astro_twilight_dawn_hour,bar_height)
			self.cr.fill()
			self.cr.stroke()
			
			pat = cairo.LinearGradient(self.width*0.9*local_sun_astro_twilight_dawn_hour+self.width*0.05,0,self.width*0.9*local_sun_astro_twilight_dawn_hour+self.width*0.05+self.width*0.9*(local_sun_rise_hour-local_sun_astro_twilight_dawn_hour),0)
			pat.add_color_stop_rgb(0.0, 0, 0, 0)
			pat.add_color_stop_rgb(1.0, 0.015625,0.515625,0.984375)

			self.cr.rectangle(self.width*0.9*local_sun_astro_twilight_dawn_hour+self.width*0.05,bar1_start,self.width*0.9*(local_sun_rise_hour-local_sun_astro_twilight_dawn_hour),bar_height)
			self.cr.set_source(pat)
			self.cr.fill()

			pat = cairo.LinearGradient(self.width*0.9*local_sun_rise_hour+self.width*0.05,0,self.width*0.9*local_sun_rise_hour+self.width*0.05+self.width*0.9*(local_sun_set_hour-local_sun_rise_hour),0)
			pat.add_color_stop_rgb(0.0, 0.015625,0.515625,0.984375)
			pat.add_color_stop_rgb(0.5, 1, 1, 1)
			pat.add_color_stop_rgb(1.0, 0.015625,0.515625,0.984375)

			self.cr.rectangle(self.width*0.9*local_sun_rise_hour+self.width*0.05,bar1_start,self.width*0.9*(local_sun_set_hour-local_sun_rise_hour),bar_height)
			self.cr.set_source(pat)
			self.cr.fill()
			
			pat = cairo.LinearGradient(self.width*0.9*local_sun_set_hour+self.width*0.05,0,self.width*0.9*local_sun_set_hour+self.width*0.05+self.width*0.9*(local_sun_astro_twilight_dusk_hour-local_sun_set_hour),0)
			pat.add_color_stop_rgb(0.0, 0.015625,0.515625,0.984375)
			pat.add_color_stop_rgb(1.0, 0, 0, 0)

			self.cr.rectangle(self.width*0.9*local_sun_set_hour+self.width*0.05,bar1_start,self.width*0.9*(local_sun_astro_twilight_dusk_hour-local_sun_set_hour),bar_height)
			self.cr.set_source(pat)
			self.cr.fill()
			
			self.cr.rectangle(self.width*0.9*local_sun_astro_twilight_dusk_hour+self.width*0.05,bar1_start,self.width*0.9*(1-local_sun_astro_twilight_dusk_hour),bar_height)
			self.cr.set_source_rgb(0.00, 0.00, 0.00)
			self.cr.fill()
			self.cr.stroke()
	
			self.cr.move_to(self.width*0.9*local_sun_antitransit_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_antitransit_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_astro_twilight_dawn_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_astro_twilight_dawn_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_naut_twilight_dawn_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_naut_twilight_dawn_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_civil_twilight_dawn_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_civil_twilight_dawn_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_rise_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_rise_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_transit_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_transit_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.set_source_rgb(0.00, 0.00, 0.00)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_set_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_set_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_civil_twilight_dusk_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_civil_twilight_dusk_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_naut_twilight_dusk_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_naut_twilight_dusk_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_astro_twilight_dusk_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_astro_twilight_dusk_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()
			
			self.cr.move_to(self.width*0.9*local_sun_now_hour+self.width*0.05,bar1_start)
			self.cr.line_to(self.width*0.9*local_sun_now_hour+self.width*0.05,bar1_start+bar_height)
			self.cr.stroke()

			self.cr.arc(self.width*0.9*local_sun_now_hour+self.width*0.05,bar1_start+bar_height/2, size, 0, 2.0*3.14)
			self.cr.set_source_rgb(1.00, 1.00, 0.00)
			self.cr.fill()
			self.cr.stroke()

			if self.gps_mode == "on" or self.gps_mode == "lock":
				if self.gps_mode == "on":
					self.cr.arc(self.width*0.9*local_sun_now_hour+self.width*0.05,bar1_start+bar_height/2, size/2, 0, 2.0*3.14)
					if self.gps_lock == True: 
						self.cr.set_source_rgb(0.00, 1.00, 0.00)
						#if self.gps_time_flag == "on":
						#	self.cr.move_to(self.width*0.9*local_sun_now_hour+self.width*0.03,bar1_start+bar_height/2+self.height*0.02)
						#	self.cr.text_path("LT")
						#else:
						#	self.cr.move_to(self.width*0.9*local_sun_now_hour+self.width*0.04,bar1_start+bar_height/2+self.height*0.02)
						#	self.cr.text_path("L")
						#self.cr.fill()
						#self.cr.stroke()
						self.cr.set_source_rgb(0.00, 1.00, 0.00)
					else:
						self.cr.set_source_rgb(1.00, 0.00, 0.00)
					self.cr.fill()
					self.cr.stroke()

				if self.gps_mode == "lock":
					self.cr.arc(self.width*0.9*local_sun_now_hour+self.width*0.05,bar1_start+bar_height/2, size/2, 0, 2.0*3.14)
					if self.gps_lock == True: 
						self.cr.set_source_rgb(1.00, 0.65, 0.00)
						#if self.gps_time_flag == "on":
						#	self.cr.move_to(self.width*0.9*local_sun_now_hour+self.width*0.03,bar1_start+bar_height/2+self.height*0.02)
						#	self.cr.text_path("LT")
						#else:
						#	self.cr.move_to(self.width*0.9*local_sun_now_hour+self.width*0.04,bar1_start+bar_height/2+self.height*0.02)
						#	self.cr.text_path("L")
						#self.cr.fill()
						#self.cr.stroke()
						self.cr.set_source_rgb(1.00, 0.65, 0.00)
					else:
						self.cr.set_source_rgb(1.00, 0.00, 0.00)
					self.cr.fill()
					self.cr.stroke()

			self.cr.arc(self.width*0.9*local_sun_now_hour+self.width*0.05,bar1_start+bar_height/2, size, 0, 2.0*3.14)
			self.cr.set_source_rgb(1.00, 0.50, 0.50)
			self.cr.set_line_width(0.8)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)

			for hour in range(0,25,1):
				sun_hour = hour/24.0
				self.cr.move_to(self.width*0.9*sun_hour+self.width*0.05,bar1_start+bar_height)
				self.cr.line_to(self.width*0.9*sun_hour+self.width*0.05,bar1_start+bar_height+tic_height)
				self.cr.set_line_width(1.0)
				self.cr.stroke()
				if hour%3 == 0:
					self.cr.move_to(self.width*0.9*sun_hour-self.width/50+self.width*0.05,bar1_start+bar_height+tic_height*2)
					self.cr.text_path(str(hour))
					self.cr.fill()
					self.cr.stroke()
		
		# day text
		if self.text_flag == "on":
			sun.compute(self.local)
			sun_alt = float(sun.alt) * 180/pi
			sun_az = float(sun.az) * 180/pi

			moon.compute(self.local)
			moon_alt = float(moon.alt) * 180/pi
			moon_az = float(moon.az) * 180/pi
				
			self.cr.set_source_rgb(1.00, 1.00, 1.00)

			self.cr.move_to(self.width*0.01,top_start)
			self.cr.text_path("LOCAL DATE:      " + str(local_sun_now)[:-9] + " " + day_of_week)
			#self.cr.text_path(self.planet_label_flag + " " + self.star_label_flag + " " + self.city_label_flag)
			#self.cr.text_path(str(self.click_count))
			#self.cr.text_path(str(moon_lat) + " " + str(moon_long))
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,top_start+text_height*2)
			self.cr.text_path("LOCAL TIME:      " + str(local_sun_now.tuple()[3]).zfill(2)+":"+str(local_sun_now.tuple()[4]).zfill(2) + " " + zone + " [UTC" + '%+2.1f' % timezone_hour + "]")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,top_start+text_height*4)
			self.cr.text_path("LOCAL LATITUDE:  " + str(self.local.lat) + u"\xb0")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,top_start+text_height*6)
			self.cr.text_path("LOCAL LONGITUDE: " + str(self.local.long) + u"\xb0")
			self.cr.fill()
			self.cr.stroke()

			if self.graphics_style != "pole":
				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*4)
				self.cr.text_path("SUN PHASE:" + sun_phase)
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*6)
				self.cr.text_path("SUN AZ:   " + str(sun.az) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*8)
				self.cr.text_path("SUN ALT:  " + str(sun.alt) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*10)
				if sun_rise_txt == "OK":
					self.cr.text_path("SUN RISE: " + str(local_sun_rise)[-8:])
				else:
					self.cr.text_path("SUN RISE: " + "N/A")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*12)
				if sun_set_txt == "OK":
					self.cr.text_path("SUN SET:  " + str(local_sun_set)[-8:])
				else:
					self.cr.text_path("SUN SET:  " + "N/A")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*14)
				self.cr.text_path("SUN MIN:  " + str(local_sun_antitransit)[-8:])
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*16)
				self.cr.text_path("SUN MAX:  " + str(local_sun_transit)[-8:])
				self.cr.fill()
				self.cr.stroke()

				if self.full_window_flag == "off":
					text_start_x = self.width*0.5
				else:	
					text_start_x = self.width*0.25

				if self.graphics_style == "info":
					self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*18+section_gap)
					if sun_astro_twilight_dawn_txt == "OK":
						self.cr.text_path("SUN AST DAWN: " + str(local_sun_astro_twilight_dawn)[-8:])
					else:
						#self.cr.text_path("SUN AST DAWN: " + sun_astro_twilight_dawn_txt)
						self.cr.text_path("SUN AST DAWN: " + "N/A")
					self.cr.fill()
					self.cr.stroke()

					self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*20+section_gap)
					if sun_naut_twilight_dawn_txt == "OK":
						self.cr.text_path("SUN NAU DAWN: " + str(local_sun_naut_twilight_dawn)[-8:])
					else:
						#self.cr.text_path("SUN NAU DAWN: " + sun_naut_twilight_dawn_txt)
						self.cr.text_path("SUN NAU DAWN: " + "N/A")
					self.cr.fill()
					self.cr.stroke()

					self.cr.move_to(self.width*0.01,bar1_start+bar_height+tic_height+text_height*22+section_gap)
					if sun_civil_twilight_dawn_txt == "OK":
						self.cr.text_path("SUN CIV DAWN: " + str(local_sun_civil_twilight_dawn)[-8:])
					else:
						#self.cr.text_path("SUN CIV DAWN: " + sun_civil_twilight_dawn_txt)
						self.cr.text_path("SUN CIV DAWN: " + "N/A")
					self.cr.fill()
					self.cr.stroke()

					self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*18+section_gap)
					if sun_astro_twilight_dawn_txt == "OK":
						self.cr.text_path("SUN AST DUSK: " + str(local_sun_astro_twilight_dusk)[-8:])
					else:
						#self.cr.text_path("SUN AST DUSK: " + sun_astro_twilight_dawn_txt)
						self.cr.text_path("SUN AST DUSK: " + "N/A")
					self.cr.fill()
					self.cr.stroke()

					self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*20+section_gap)
					if sun_naut_twilight_dusk_txt == "OK":
						self.cr.text_path("SUN NAU DUSK: " + str(local_sun_naut_twilight_dusk)[-8:])
					else:
						#self.cr.text_path("SUN NAU DUSK: " + sun_naut_twilight_dusk_txt)
						self.cr.text_path("SUN NAU DUSK: " + "N/A")
					self.cr.fill()
					self.cr.stroke()

					self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*22+section_gap)
					if sun_civil_twilight_dusk_txt == "OK":
						self.cr.text_path("SUN CIV DUSK: " + str(local_sun_civil_twilight_dusk)[-8:])
					else:
						#self.cr.text_path("SUN CIV DUSK: " + sun_civil_twilight_dusk_txt)
						self.cr.text_path("SUN CIV DUSK: " + "N/A")
					self.cr.fill()
					self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*4)
				self.cr.text_path("DAYLENGTH: " + str(day_length) + " hrs")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*6)
				self.cr.text_path("MOON AZ:  " + str(moon.az) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*8)
				self.cr.text_path("MOON ALT: " + str(moon.alt) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*10)
				if moon_rise_txt == "OK":
					self.cr.text_path("MOON RISE:" + str(local_moon_rise)[-8:])
				else:
					self.cr.text_path("MOON RISE:" + "N/A")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*12)
				if moon_set_txt == "OK":
					self.cr.text_path("MOON SET: " + str(local_moon_set)[-8:])
				else:
					self.cr.text_path("MOON SET: " + "N/A")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*14)
				self.cr.text_path("MOON MIN: " + str(local_moon_antitransit)[-8:])
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(text_start_x,bar1_start+bar_height+tic_height+text_height*16)
				self.cr.text_path("MOON MAX: " + str(local_moon_transit)[-8:])
				self.cr.fill()
				self.cr.stroke()

			else:
				self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*4)
				self.cr.text_path("SIDEREAL TIME:" + str(self.local.sidereal_time()))
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*6)
				self.cr.text_path("POLARIS AZ:   " + str(star.az) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*8)
				self.cr.text_path("POLARIS ALT:  " + str(star.alt) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*10)
				self.cr.text_path("POLARIS RA:  " + str(star.ra) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

				self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*12)
				self.cr.text_path("POLARIS DEC: " + str(star.dec) + u"\xb0")
				self.cr.fill()
				self.cr.stroke()

		if self.graphics_style == "info":
			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			text_height = self.height * 0.013 * screen_ratio
			font_size = 25
			self.cr.set_font_size(font_size)

			line_len = 21

			self.cr.move_to(self.width*0.01, text_height*6)
			#self.cr.text_path(string.center(str(local_sun_now)[:-9] + " " + day_of_week, line_len))
			self.cr.text_path(string.center(str(local_sun_now)[:-9] + " GD", line_len))
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01, text_height*10)
			self.cr.text_path(string.center(str(ephem.julian_date()) + " JD", line_len))
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,text_height*16)
			self.cr.text_path(string.center(str(local_sun_now.tuple()[3]).zfill(2) + ":" + str(local_sun_now.tuple()[4]).zfill(2) + " " + zone, line_len))
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,text_height*20)
			self.cr.text_path(string.center(str(self.local.sidereal_time()) + " LST", line_len))
			self.cr.fill()
			self.cr.stroke()

			font_size = 11
			self.cr.set_font_size(font_size)

		# season section

		self.local.date = save

		current_year = self.local.date.tuple()[0]
		first_year_day = ephem.date(str(current_year)+"/01/01")
		last_year_day = ephem.date(str(current_year+1)+"/01/01")
		days_in_year = last_year_day-first_year_day
		local_date = ephem.localtime(self.local.date)
		current_year_day = ephem.date(local_date) - first_year_day
		local_year_day = current_year_day / days_in_year
	
		spring_date = ephem.localtime(ephem.next_equinox(ephem.date(str(first_year_day.tuple()[0]))))
		summer_date = ephem.localtime(ephem.next_solstice(spring_date))
		fall_date = ephem.localtime(ephem.next_equinox(summer_date))
		winter_date = ephem.localtime(ephem.next_solstice(fall_date))
		spring_year_day = (ephem.date(spring_date) - first_year_day) / days_in_year
		summer_year_day = (ephem.date(summer_date) - first_year_day) / days_in_year
		fall_year_day = (ephem.date(fall_date) - first_year_day) / days_in_year
		winter_year_day = (ephem.date(winter_date) - first_year_day) / days_in_year

		if (local_year_day >= spring_year_day) and (local_year_day < summer_year_day):
			season = "Spring"
		if (local_year_day >= summer_year_day) and (local_year_day < fall_year_day):
			season = "Summer"
		if (local_year_day >= fall_year_day) and (local_year_day < winter_year_day):
			season = "Fall"
		if (local_year_day >= winter_year_day) or (local_year_day < spring_year_day):
			season = "Winter"
			
		month_year_day = ([])
	
		jan_date = ephem.date(str(current_year)+"/01/01")
		feb_date = ephem.date(str(current_year)+"/02/01")	
		mar_date = ephem.date(str(current_year)+"/03/01")	
		apr_date = ephem.date(str(current_year)+"/04/01")	
		may_date = ephem.date(str(current_year)+"/05/01")	
		jun_date = ephem.date(str(current_year)+"/06/01")	
		jul_date = ephem.date(str(current_year)+"/07/01")	
		aug_date = ephem.date(str(current_year)+"/08/01")	
		sep_date = ephem.date(str(current_year)+"/09/01")	
		oct_date = ephem.date(str(current_year)+"/10/01")	
		nov_date = ephem.date(str(current_year)+"/11/01")	
		dec_date = ephem.date(str(current_year)+"/12/01")	

		jan_year_day = (jan_date - first_year_day + 1) / days_in_year
		month_year_day.append([jan_year_day, "J"])
		feb_year_day = (feb_date - first_year_day + 1) / days_in_year
		month_year_day.append([feb_year_day, "F"])
		mar_year_day = (mar_date - first_year_day + 1) / days_in_year
		month_year_day.append([mar_year_day, "M"])
		apr_year_day = (apr_date - first_year_day + 1) / days_in_year
		month_year_day.append([apr_year_day, "A"])
		may_year_day = (may_date - first_year_day + 1) / days_in_year
		month_year_day.append([may_year_day, "M"])
		jun_year_day = (jun_date - first_year_day + 1) / days_in_year
		month_year_day.append([jun_year_day, "J"])
		jul_year_day = (jul_date - first_year_day + 1) / days_in_year
		month_year_day.append([jul_year_day, "J"])
		aug_year_day = (aug_date - first_year_day + 1) / days_in_year
		month_year_day.append([aug_year_day, "A"])
		sep_year_day = (sep_date - first_year_day + 1) / days_in_year
		month_year_day.append([sep_year_day, "S"])
		oct_year_day = (oct_date - first_year_day + 1) / days_in_year
		month_year_day.append([oct_year_day, "O"])
		nov_year_day = (nov_date - first_year_day + 1) / days_in_year
		month_year_day.append([nov_year_day, "N"])
		dec_year_day = (dec_date - first_year_day + 1) / days_in_year
		month_year_day.append([dec_year_day, "D"])

		self.cr.set_source_rgb(1.00, 1.00, 1.00)

		# season bars
		if self.graphics_style == "bar":
			pat = cairo.LinearGradient(self.width*0.05,0,self.width*0.05+self.width*0.9,0)
			pat.add_color_stop_rgb(0.00, 1.00, 1.00, 1.00)
			pat.add_color_stop_rgb(spring_year_day, 0.00, 1.00, 0.00)
			pat.add_color_stop_rgb(summer_year_day, 1.00, 0.00, 0.00)
			pat.add_color_stop_rgb(fall_year_day, 1.00, 1.00, 0.00)
			pat.add_color_stop_rgb(winter_year_day, 1.00, 1.00, 1.00)

			self.cr.rectangle(self.width*0.05,bar2_start,self.width*0.9,bar_height)
			self.cr.set_source(pat)
			self.cr.fill()

			#self.cr.set_dash([1],1) 	
			self.cr.set_line_width(1.0)
			self.cr.set_source_rgb(0.00, 0.00, 0.00)

			self.cr.move_to(self.width*0.9*spring_year_day+self.width*0.05,bar2_start)
			self.cr.line_to(self.width*0.9*spring_year_day+self.width*0.05,bar2_start+bar_height)
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*summer_year_day+self.width*0.05,bar2_start)
			self.cr.line_to(self.width*0.9*summer_year_day+self.width*0.05,bar2_start+bar_height)
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*fall_year_day+self.width*0.05,bar2_start)
			self.cr.line_to(self.width*0.9*fall_year_day+self.width*0.05,bar2_start+bar_height)
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*winter_year_day+self.width*0.05,bar2_start)
			self.cr.line_to(self.width*0.9*winter_year_day+self.width*0.05,bar2_start+bar_height)
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*local_year_day+self.width*0.05,bar2_start)
			self.cr.line_to(self.width*0.9*local_year_day+self.width*0.05,bar2_start+bar_height)
			self.cr.stroke()

			self.cr.arc(self.width*0.9*local_year_day+self.width*0.05,bar2_start+bar_height/2, size, 0, 2.0*3.14)
			self.cr.set_source_rgb(0.015625,0.515625,0.984375) # light blue dome color
			self.cr.fill()
			self.cr.stroke()
		
			self.cr.arc(self.width*0.9*local_year_day+self.width*0.05,bar2_start+bar_height/2, size, 0, 2.0*3.14)
			self.cr.set_source_rgb(0.00, 0.00, 1.00)
			self.cr.set_line_width(0.5)
			self.cr.stroke()
	
			for month_start in month_year_day:
				self.cr.move_to(self.width*0.9*month_start[0]+self.width*0.05,bar2_start+bar_height)
				self.cr.line_to(self.width*0.9*month_start[0]+self.width*0.05,bar2_start+bar_height+tic_height)
				self.cr.stroke()

				self.cr.move_to(self.width*0.9*month_start[0]+self.width/30+self.width*0.05,bar2_start+bar_height+tic_height*2)
				self.cr.text_path(month_start[1])
				self.cr.set_source_rgb(1.00, 1.00, 1.00)
				self.cr.fill()
				self.cr.stroke()

		# season circle
		if self.graphics_style == "sphere":
			radius = sun_year_circle
			radius2 = sun_year_circle - circ_diff

			red_inc = 1
			green_inc = 1 
			blue_inc = 1

			for mult_pos in range(0, int(spring_year_day*10000), int((spring_year_day)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (spring_year_day)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc - 0.1
				green_inc = green_inc - 0
				blue_inc = blue_inc - 0.1
				self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 0
			green_inc = 1 
			blue_inc = 0

			for mult_pos in range(int(spring_year_day*10000), int(summer_year_day*10000), int((summer_year_day-spring_year_day)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (summer_year_day-spring_year_day)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc + 0.1
				green_inc = green_inc - 0.1
				blue_inc = blue_inc - 0
				self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 1
			green_inc = 0 
			blue_inc = 0

			for mult_pos in range(int(summer_year_day*10000), int(fall_year_day*10000), int((fall_year_day-summer_year_day)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (fall_year_day-summer_year_day)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc + 0
				green_inc = green_inc + 0.1
				blue_inc = blue_inc - 0
				self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 1
			green_inc = 1 
			blue_inc = 0

			for mult_pos in range(int(fall_year_day*10000), int(winter_year_day*10000), int((winter_year_day-fall_year_day)*1000)):
				start_pos = mult_pos/10000.0
				stop_pos = start_pos + (winter_year_day-fall_year_day)/10
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc + 0
				green_inc = green_inc + 0
				blue_inc = blue_inc + 0.1
				self.pie_slice(start_pos, stop_pos, radius, color)

			start_pos = winter_year_day
			stop_pos = 1
			color = [1, 1, 1]
			self.pie_slice(start_pos, stop_pos, radius, color)

			pos = spring_year_day
			color = [0.00, 0.00, 0.00]
			self.radial_line(pos, radius, color)

			pos = summer_year_day
			color = [1.00, 1.00, 1.00]
			self.radial_line(pos, radius, color)

			pos = fall_year_day
			color = [0.00, 0.00, 0.00]
			self.radial_line(pos, radius, color)

			pos = winter_year_day
			self.radial_line(pos, radius, color)

			pos = local_year_day
			color = [0.015625,0.515625,0.984375]
			self.radial_ball(pos, radius2, size, color, True)

			pos = local_year_day
			color = [0.00, 0.00, 1.00]
			self.radial_ball(pos, radius2, size, color, False)

		# season text info
		if self.text_flag == "on" and self.graphics_style not in ["earth", "moon", "pole", "horizon"]:
			self.cr.set_source_rgb(1.00, 1.00, 1.00)

			self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*4)
			self.cr.text_path("SEASON/DAY: " + season + " / " + "%1.2f" % current_year_day + " [" + str(int(current_year_day)+1) +"]")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*6)
			self.cr.text_path("SPRING:     " + str(spring_date)[:19])
			#self.cr.text_path(str(os.path.expanduser("~")) + "/.mchronos/mchronos.ini")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*8)
			self.cr.text_path("SUMMER:     " + str(summer_date)[:19])
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*10)
			self.cr.text_path("FALL:       " + str(fall_date)[:19])
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar2_start+bar_height+tic_height+text_height*12)
			self.cr.text_path("WINTER:     " + str(winter_date)[:19])
			self.cr.fill()
			self.cr.stroke()

		# moon section

		moon = ephem.Moon()
		moon.compute(self.local)
		moon_full_pct = moon.phase

		nm = 0.00
		fq = 0.25
		fm = 0.50
		lq = 0.75
		nm2 = 1.00
	
		# dome circle
		if self.graphics_style == "dome":
			moon.compute(self.local)
			moon_alt = float(moon.alt) * 180/pi
			moon_az = self.astro_sign * float(moon.az) * 180/pi
			radius = sun_day_circle
			radius2 = sun_day_circle * ((90 - moon_alt) / 90.0)

		# moon circle
		if self.graphics_style == "sphere":
			radius = moon_month_circle
			radius2 = moon_month_circle - circ_diff

			red_inc = 0
			green_inc = 0 
			blue_inc = 0

			for mult_pos in range(0, 50, 5):
				start_pos = mult_pos/100.0
				stop_pos = start_pos + 0.1
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc + 0.1
				green_inc = green_inc + 0.1
				blue_inc = blue_inc + 0.1
				self.pie_slice(start_pos, stop_pos, radius, color)

			red_inc = 1
			green_inc = 1 
			blue_inc = 1

			for mult_pos in range(50, 100, 5):
				start_pos = mult_pos/100.0
				stop_pos = start_pos + 0.1
				color = [red_inc, green_inc, blue_inc]
				red_inc = red_inc - 0.1
				green_inc = green_inc - 0.1
				blue_inc = blue_inc - 0.1
				self.pie_slice(start_pos, stop_pos, radius, color)

			pos = nm
			color = [1.00, 1.00, 1.00]
			self.radial_line(pos, radius, color)

			pos = fq
			self.radial_line(pos, radius, color)

			pos = fm
			color = [0.00, 0.00, 0.00]
			self.radial_line(pos, radius, color)

			pos = lq
			color = [1.00, 1.00, 1.00]
			self.radial_line(pos, radius, color)

			pos = nm2
			self.radial_line(pos, radius, color)

		# moon bar
		if self.graphics_style in ["bar", "moon"]:
			pat = cairo.LinearGradient(self.width*0.05,bar3_start,self.width*.90+self.width*0.05,bar3_start)
			pat.add_color_stop_rgb(0.0, 0, 0, 0)
			pat.add_color_stop_rgb(0.5, 1, 1, 1)
			pat.add_color_stop_rgb(1.0, 0, 0, 0)

			self.cr.rectangle(self.width*0.05,bar3_start,self.width*0.90,bar_height)
			self.cr.set_source(pat)
			self.cr.fill()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*nm+self.width*0.05,bar3_start)
			self.cr.line_to(self.width*0.9*nm+self.width*0.05,bar3_start+bar_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*fq+self.width*0.05,bar3_start)
			self.cr.line_to(self.width*0.9*fq+self.width*0.05,bar3_start+bar_height)
			self.cr.stroke()

			self.cr.set_source_rgb(0.00, 0.00, 0.00)
			self.cr.move_to(self.width*0.9*fm+self.width*0.05,bar3_start)
			self.cr.line_to(self.width*0.9*fm+self.width*0.05,bar3_start+bar_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*lq+self.width*0.05,bar3_start)
			self.cr.line_to(self.width*0.9*lq+self.width*0.05,bar3_start+bar_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*nm2+self.width*0.05,bar3_start)
			self.cr.line_to(self.width*0.9*nm2+self.width*0.05,bar3_start+bar_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*nm+self.width*0.05,bar3_start+bar_height)
			self.cr.line_to(self.width*0.9*nm+self.width*0.05,bar3_start+bar_height+tic_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*fq+self.width*0.05,bar3_start+bar_height)
			self.cr.line_to(self.width*0.9*fq+self.width*0.05,bar3_start+bar_height+tic_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*fm+self.width*0.05,bar3_start+bar_height)
			self.cr.line_to(self.width*0.9*fm+self.width*0.05,bar3_start+bar_height+tic_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*lq+self.width*0.05,bar3_start+bar_height)
			self.cr.line_to(self.width*0.9*lq+self.width*0.05,bar3_start+bar_height+tic_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(self.width*0.9*nm2+self.width*0.05,bar3_start+bar_height)
			self.cr.line_to(self.width*0.9*nm2+self.width*0.05,bar3_start+bar_height+tic_height)
			self.cr.stroke()

			self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.set_line_width(1.0)

			self.cr.move_to(self.width*0.9*nm-self.width/50+self.width*0.05,bar3_start+bar_height+tic_height*2)
			self.cr.text_path("NM")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*fq-self.width/50+self.width*0.05,bar3_start+bar_height+tic_height*2)
			self.cr.text_path("FQ")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*fm-self.width/50+self.width*0.05,bar3_start+bar_height+tic_height*2)
			self.cr.text_path("FM")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*lq-self.width/50+self.width*0.05,bar3_start+bar_height+tic_height*2)
			self.cr.text_path("LQ")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.9*nm2-self.width/50+self.width*0.05,bar3_start+bar_height+tic_height*2)
			self.cr.text_path("NM")
			self.cr.fill()
			self.cr.stroke()

		save = self.local.date

		self.local.date = start_of_day+timezone_offset # compensate for differing time zone

		moon_next_full = ephem.next_full_moon(self.local.date)
		moon_next_new = ephem.next_new_moon(self.local.date)

		self.local.date = save

		if self.graphics_style == "moon":
			if self.full_window_flag == "off":
				image = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_moon_250x200.png")
				image2 = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_moon_250x200.png")
			else:
				image = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_moon_720x400.png")
				image2 = cairo.ImageSurface.create_from_png(os.path.expanduser("~") + "/.mchronos/mchronos_moon_720x400.png")

			self.cr.set_source_surface(image, 0, 0)
			self.cr.paint()

			image_width = image.get_width()
			image_height = image.get_height()

			size = image_height / 2.0

		# moon special bar & circle & moon & dome
		if self.graphics_style in ["bar", "sphere", "moon", "dome", "earth", "horizon"]:
			if not (self.graphics_style  == "dome" and moon_alt < 0):
				if moon_next_full < moon_next_new:
					mp = (moon_full_pct/100)/2
					moon_phase = "Waxing"
					arc_start = True

					if self.graphics_style == "horizon":
						x = (moon_az / 360.0) * self.width
						y = ((90 -moon_alt) / 90.0) * (self.width / 4.0)
						self.cr.set_source_rgb(0.3, 0.3, 0.3)
						self.cr.arc(x, y, size, 0, 2 * math.pi)
						self.cr.fill()
						self.cr.stroke()

					if self.graphics_style == "earth":
						x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0)
						y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
						self.cr.set_source_rgb(0.3, 0.3, 0.3)
						self.cr.arc(x, y, size, 0, 2 * math.pi)
						self.cr.fill()
						self.cr.stroke()

					if self.graphics_style == "dome":
						pos = (moon_az - 180.0) / 360.0
						color = [0.3, 0.3, 0.3]
						self.radial_ball(pos, radius2, size, color, True)
						color = [1, 1, 1]
						self.radial_ball(pos, radius2, size, color, False)
						x1 = self.circle_x_center
						y1 = self.circle_y_center
						angle1a = pos * 360
						angle1b = (angle1a+90)*pi/180

					if self.graphics_style == "bar":
						self.cr.set_source_rgb(1.00, 0.00, 0.00)
						self.cr.move_to(self.width*0.9*mp+self.width*0.05,bar3_start)
						self.cr.line_to(self.width*0.9*mp+self.width*0.05,bar3_start+bar_height)
						self.cr.set_line_width(1.0)
						self.cr.stroke()

						self.cr.arc(self.width*0.9*mp+self.width*0.05,bar3_start+bar_height/2, size, 0, 2.0*3.14)
						self.cr.set_source_rgb(0.00, 0.00, 0.00)
						self.cr.fill()
						self.cr.stroke()

						self.cr.arc(self.width*0.9*mp+self.width*0.05,bar3_start+bar_height/2, size, 0, 2.0*3.14)
						if moon_full_pct < 25 or moon_full_pct > 75:
							self.cr.set_source_rgb(1.00, 1.00, 1.00)
						else:
							self.cr.set_source_rgb(0.00, 0.00, 0.00)
						self.cr.set_line_width(0.5)
						self.cr.stroke()

					if self.graphics_style == "sphere":
						pos = mp
						color = [0, 0, 0]
						self.radial_ball(pos, radius2, size, color, True)
						color = [1, 1, 1]
						self.radial_ball(pos, radius2, size, color, False)
						x1 = self.circle_x_center
						y1 = self.circle_y_center
						angle1a = pos * 360
						angle1b = (angle1a+90)*pi/180

					view_az = 0
					view_alt = 0
					if self.graphics_style == "moon":
						az_degrees = -90
					else:
						az_degrees = 90
					az = az_degrees * pi/180
					for alt_degrees in range(90,-90,-9):
						alt = alt_degrees * pi/180
						arc_x = size * cos(alt)*sin(az-view_az)
						arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
						if self.graphics_style == "moon":
							x = ((image_width / 2.0) + arc_x)
							y = ((image_height / 2.0) - arc_y)
						if self.graphics_style == "bar":
							x = ((self.width*0.9*mp+self.width*0.05) + arc_x)
							y = ((bar3_start+bar_height/2) - arc_y)
						if self.graphics_style == "sphere":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "dome":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "earth":
							x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0) + arc_x
							y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) - arc_y
						if self.graphics_style == "horizon":
							x = (moon_az / 360.0) * self.width + arc_x
							y = ((90 -moon_alt) / 90.0) * (self.width / 4.0) - arc_y
						if arc_start == True:
							self.cr.move_to(x,y)
							arc_start = False
						else:
							self.cr.line_to(x,y)

					view_az = 0
					view_alt = 0
					az_degrees = -asin((moon_full_pct - 50)/50) * 180/pi

					az = az_degrees * pi/180
					for alt_degrees in range(-90,90,9):
						alt = alt_degrees * pi/180
						arc_x = size * cos(alt)*sin(az-view_az)
						arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
						if self.graphics_style == "moon":
							x = ((image_width / 2.0) + arc_x)
							y = ((image_height / 2.0) - arc_y)
						if self.graphics_style == "bar":
							x = ((self.width*0.9*mp+self.width*0.05) + arc_x)
							y = ((bar3_start+bar_height/2) - arc_y)
						if self.graphics_style == "sphere":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "dome":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "earth":
							x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0) + arc_x
							y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) - arc_y
						if self.graphics_style == "horizon":
							x = (moon_az / 360.0) * self.width + arc_x
							y = ((90 -moon_alt) / 90.0) * (self.width / 4.0) - arc_y
						self.cr.line_to(x,y)

				else:
					mp = (1-(moon_full_pct/100))/2+0.5
					moon_phase = "Waning"
					arc_start = True

					if self.graphics_style == "horizon":
						x = (moon_az / 360.0) * self.width
						y = ((90 -moon_alt) / 90.0) * (self.width / 4.0)
						self.cr.set_source_rgb(0.3, 0.3, 0.3)
						self.cr.arc(x, y, size, 0, 2 * math.pi)
						self.cr.fill()
						self.cr.stroke()

					if self.graphics_style == "earth":
						x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0)
						y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) 
						self.cr.set_source_rgb(0.3, 0.3, 0.3)
						self.cr.arc(x, y, size, 0, 2 * math.pi)
						self.cr.fill()
						self.cr.stroke()

					if self.graphics_style == "dome":
						pos = (moon_az - 180.0) / 360.0
						color = [0.3, 0.3, 0.3]
						self.radial_ball(pos, radius2, size, color, True)
						color = [1, 1, 1]
						self.radial_ball(pos, radius2, size, color, False)
						x1 = self.circle_x_center
						y1 = self.circle_y_center
						angle1a = pos * 360
						angle1b = (angle1a+90)*pi/180

					if self.graphics_style == "bar":
						self.cr.set_source_rgb(1.00, 0.00, 0.00)
						self.cr.move_to(self.width*0.9*mp+self.width*0.05,bar3_start)
						self.cr.line_to(self.width*0.9*mp+self.width*0.05,bar3_start+bar_height)
						self.cr.set_line_width(1.0)
						self.cr.stroke()

						self.cr.arc(self.width*0.9*mp+self.width*0.05,bar3_start+bar_height/2, size, 0, 2.0*3.14)
						self.cr.set_source_rgb(0.00, 0.00, 0.00)
						self.cr.fill()
						self.cr.stroke()

						self.cr.arc(self.width*0.9*mp+self.width*0.05,bar3_start+bar_height/2, size, 0, 2.0*3.14)
						if moon_full_pct < 25 or moon_full_pct > 75:
							self.cr.set_source_rgb(1.00, 1.00, 1.00)
						else:
							self.cr.set_source_rgb(0.00, 0.00, 0.00)
						self.cr.set_line_width(0.5)
						self.cr.stroke()

					if self.graphics_style == "sphere":
						pos = mp
						color = [0, 0, 0]
						self.radial_ball(pos, radius2, size, color, True)
						color = [1, 1, 1]
						self.radial_ball(pos, radius2, size, color, False)
						x1 = self.circle_x_center
						y1 = self.circle_y_center
						angle1a = pos * 360
						angle1b = (angle1a+90)*pi/180

					view_az = 0
					view_alt = 0
					if self.graphics_style == "moon":
						az_degrees = 90
					else:
						az_degrees = -90
					az = az_degrees * pi/180
					for alt_degrees in range(90,-90,-9):
						alt = alt_degrees * pi/180
						arc_x = size * cos(alt)*sin(az-view_az)
						arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
						if self.graphics_style == "moon":
							x = ((image_width / 2.0) + arc_x)
							y = ((image_height / 2.0) - arc_y)
						if self.graphics_style == "bar":
							x = ((self.width*0.9*mp+self.width*0.05) + arc_x)
							y = ((bar3_start+bar_height/2) - arc_y)
						if self.graphics_style == "sphere":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "dome":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "earth":
							x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0) + arc_x
							y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) - arc_y
						if self.graphics_style == "horizon":
							x = (moon_az / 360.0) * self.width + arc_x
							y = ((90 -moon_alt) / 90.0) * (self.width / 4.0) - arc_y
						if arc_start == True:
							self.cr.move_to(x,y)
							arc_start = False
						else:
							self.cr.line_to(x,y)

					view_az = 0
					view_alt = 0
					az_degrees = asin((moon_full_pct - 50)/50) * 180/pi
					az = az_degrees * pi/180
					for alt_degrees in range(-90,90,9):
						alt = alt_degrees * pi/180
						arc_x = size * cos(alt)*sin(az-view_az)
						arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
						if self.graphics_style == "moon":
							x = ((image_width / 2.0) + arc_x)
							y = ((image_height / 2.0) - arc_y)
						if self.graphics_style == "bar":
							x = ((self.width*0.9*mp+self.width*0.05) + arc_x)
							y = ((bar3_start+bar_height/2) - arc_y)
						if self.graphics_style == "sphere":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "dome":
							x = (x1 + (radius2 * math.cos(angle1b)) + arc_x)
							y = (y1 + (radius2 * math.sin(angle1b)) - arc_y)
						if self.graphics_style == "earth":
							x = (moon_long/180.0) * (image_width/2.0) + (image_width/2.0) + arc_x
							y = ((moon_lat/90.0) * (image_height/2.0) + (image_height/2.0)) - arc_y
						if self.graphics_style == "horizon":
							x = (moon_az / 360.0) * self.width + arc_x
							y = ((90 -moon_alt) / 90.0) * (self.width / 4.0) - arc_y
						self.cr.line_to(x,y)

				if self.graphics_style in ["bar", "sphere", "dome", "earth", "horizon"]:
					self.cr.set_source_rgb(1,1,1)

				if self.graphics_style == "moon":
					self.cr.set_source_rgb(0,0,0)

				self.cr.paint_with_alpha(0.1)
				self.cr.fill()
				self.cr.stroke()
				self.cr.close_path()

				if self.graphics_style == "moon":
					self.cr.set_source_surface(image2, 0, 0)
					self.cr.paint_with_alpha(0.3)

					if self.moon_grid_flag == "on":
						self.cr.set_line_width(0.1)
						self.cr.set_source_rgb(0.0, 1.0, 0.0)
						view_az = 0
						view_alt = 0
						for az_degrees in range(-90, 90, 15):
							az = az_degrees * pi/180
							for alt_degrees in range(-90, 90, 3):
								alt = alt_degrees * pi/180
								arc_x = size * cos(alt)*sin(az-view_az)
								arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
								x = ((image_width / 2.0) + arc_x)
								y = ((image_height / 2.0) - arc_y)
								self.cr.move_to(x, y)
								alt = (alt_degrees + 15) * pi/180
								arc_x = size * cos(alt)*sin(az-view_az)
								arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
								x = ((image_width / 2.0) + arc_x)
								y = ((image_height / 2.0) - arc_y)
								self.cr.line_to(x, y)
								self.cr.stroke()
						self.cr.set_line_width(0.3)
						for alt_degrees in range(-90, 90, 15):
							alt = alt_degrees * pi/180
							az_degrees = -90
							az = az_degrees * pi/180
							arc_x = size * cos(alt)*sin(az-view_az)
							arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
							x = ((image_width / 2.0) + arc_x)
							y = ((image_height / 2.0) - arc_y)
							self.cr.move_to(x, y)
							az_degrees = 90
							az = az_degrees * pi/180
							arc_x = size * cos(alt)*sin(az-view_az)
							arc_y = size * (cos(view_alt)*sin(alt)-sin(view_alt)*cos(alt)*cos(az-view_az))
							x = ((image_width / 2.0) + arc_x)
							y = ((image_height / 2.0) - arc_y)
							self.cr.line_to(x, y)
							self.cr.stroke()

		moon_previous_new = ephem.localtime(ephem.previous_new_moon(self.local.date))
		moon_previous_first_quarter = ephem.localtime(ephem.previous_first_quarter_moon(self.local.date))
		moon_previous_full = ephem.localtime(ephem.previous_full_moon(self.local.date))
		moon_previous_last_quarter = ephem.localtime(ephem.previous_last_quarter_moon(self.local.date))

		moon_next_last_quarter = ephem.localtime(ephem.next_last_quarter_moon(self.local.date))
		moon_next_new = ephem.localtime(ephem.next_new_moon(self.local.date))
		moon_next_first_quarter = ephem.localtime(ephem.next_first_quarter_moon(self.local.date))
		moon_next_full = ephem.localtime(ephem.next_full_moon(self.local.date))
	
		moon_age = self.local.date - ephem.date(moon_previous_new)

		if self.text_flag == "on" and self.graphics_style not in ["pole"]:
			self.cr.set_source_rgb(1.00, 1.00, 1.00)

			self.cr.set_line_width(1.0)
			self.cr.set_source_rgb(1.00, 1.00, 1.00)

			self.cr.move_to(self.width*0.01,bar3_start+bar_height+tic_height+text_height*4)
			self.cr.text_path("MOON FULL/AGE: " + str(round(moon_full_pct)) + "%" + " / " + "%1.1f" % moon_age + " days")
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar3_start+bar_height+tic_height+text_height*6)
			self.cr.text_path("NEW MOON:      " + str(moon_next_new)[:19])
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar3_start+bar_height+tic_height+text_height*8)
			self.cr.text_path("FIRST QUARTER: " + str(moon_next_first_quarter)[:19])
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar3_start+bar_height+tic_height+text_height*10)
			self.cr.text_path("FULL MOON:     " + str(moon_next_full)[:19])
			self.cr.fill()
			self.cr.stroke()

			self.cr.move_to(self.width*0.01,bar3_start+bar_height+tic_height+text_height*12)
			self.cr.text_path("LAST QUARTER:  " + str(moon_next_last_quarter)[:19])
			self.cr.fill()
			self.cr.stroke()

		#toggle_x = 0
		toggle_x = 25

		if self.text_flag == "on":
			self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
		else:
                        self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
		self.cr.rectangle(toggle_x, self.height-20, 20, 20)
		self.cr.fill()
		self.cr.stroke()
                self.cr.set_source_rgb(1.00, 1.00, 1.00)
		self.cr.move_to(toggle_x + 5, self.height-7)
		self.cr.text_path("T")
		self.cr.fill()
		self.cr.stroke()

		toggle_x = toggle_x + 25
		if self.full_window_flag == "on":
			self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
		else:
                        self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
		self.cr.rectangle(toggle_x, self.height-20, 20, 20)
		self.cr.fill()
		self.cr.stroke()
                self.cr.set_source_rgb(1.00, 1.00, 1.00)
		self.cr.move_to(toggle_x + 5, self.height-7)
		self.cr.text_path("F")
		self.cr.fill()
		self.cr.stroke()

		toggle_x = toggle_x + 25
		if self.gps_mode in ["on", "lock"]:
			self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
		else:
                        self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
		self.cr.rectangle(toggle_x, self.height-20, 20, 20)
		self.cr.fill()
		self.cr.stroke()
                self.cr.set_source_rgb(1.00, 1.00, 1.00)
		self.cr.move_to(toggle_x + 5, self.height-7)
		self.cr.text_path("G")
		self.cr.fill()
		self.cr.stroke()

                if len(self.graphics_style_all) >= 2: # do not draw label if only one choice
			choice_x = toggle_x + 30
			if self.graphics_style == self.graphics_style_all[0]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[0][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[1]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[1][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                if len(self.graphics_style_all) >= 3:
			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[2]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[2][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                if len(self.graphics_style_all) >= 4:
			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[3]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[3][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                if len(self.graphics_style_all) >= 5:
			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[4]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[4][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                if len(self.graphics_style_all) >= 6:
			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[5]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[5][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                if len(self.graphics_style_all) >= 7:
			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[6]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[6][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                if len(self.graphics_style_all) >= 8:
			choice_x = choice_x + 25
			if self.graphics_style == self.graphics_style_all[7]:
				self.cr.set_source_rgba(0.00, 1.00, 0.00, 0.50)
			else:
                        	self.cr.set_source_rgba(0.00, 0.00, 1.00, 0.50) 
                        self.cr.rectangle(choice_x, self.height-20, 20, 20)
                        self.cr.fill()
                        self.cr.stroke()
                	self.cr.set_source_rgb(1.00, 1.00, 1.00)
			self.cr.move_to(choice_x + 5, self.height-7)
                        label = string.upper(self.graphics_style_all[7][:1])
                        self.cr.text_path(label)
                        self.cr.fill()
                        self.cr.stroke()

                self.cr.set_source_rgb(1.00, 1.00, 1.00)

		return False

	def pie_slice(self, start_pos, stop_pos, radius, color):

		angle1a = start_pos * 360
		angle1b = (angle1a+90)*pi/180

		self.cr.set_source_rgb(color[0], color[1], color[2])

		self.cr.move_to(self.circle_x_center, self.circle_y_center)
		self.cr.rel_line_to(radius * math.cos(angle1b), radius * math.sin(angle1b))

		angle2a = stop_pos * 360
		angle2b = (angle2a+90)*pi/180

		self.cr.arc(self.circle_x_center, self.circle_y_center, radius, angle1b, angle2b)

		self.cr.move_to(self.circle_x_center, self.circle_y_center)

		self.cr.fill()
		self.cr.stroke()
		self.cr.close_path()

		return

	def radial_line(self, pos, radius, color):

		angle1a = pos * 360
		angle1b = (angle1a+90)*pi/180

		self.cr.set_source_rgb(color[0], color[1], color[2])

		self.cr.move_to(self.circle_x_center, self.circle_y_center)
		self.cr.rel_line_to(radius * math.cos(angle1b), radius * math.sin(angle1b))
		self.cr.stroke()

		return

	def radial_ball(self, pos, radius1, radius2, color, fill):

		angle1a = pos * 360
		angle1b = (angle1a+90)*pi/180

		self.cr.set_source_rgb(color[0], color[1], color[2])
		self.cr.arc(self.circle_x_center + (radius1 * math.cos(angle1b)), self.circle_y_center + (radius1 * math.sin(angle1b)), radius2, 0, 2.0*3.14)
		if fill == True:
			self.cr.fill()	
		self.cr.stroke()

		return

	def radial_text(self, pos, radius1, color, text):

		angle1a = pos * 360
		angle1b = (angle1a+90)*pi/180

		self.cr.set_source_rgb(color[0], color[1], color[2])
		self.cr.move_to(self.circle_x_center + (radius1 * math.cos(angle1b)), self.circle_y_center + (radius1 * math.sin(angle1b)))
		self.cr.text_path(text)
		self.cr.fill()	
		self.cr.stroke()

		return

	def resize_window(self):

		if self.text_flag == "on":
			if self.graphics_style == "info":
				#self.set_size_request(330, 300)
				self.set_size_request(330, 400)
			if self.graphics_style == "bar":
				self.set_size_request(330, 400)
			if self.graphics_style == "sphere":
				self.set_size_request(330, 400)
			if self.graphics_style == "dome":
				self.set_size_request(330, 400)
			if self.graphics_style == "moon":
				self.set_size_request(330, 290)
			if self.graphics_style == "earth":
				self.set_size_request(400, 400)
			if self.graphics_style == "horizon":
				self.set_size_request(400, 400)
			if self.graphics_style == "pole":
				self.set_size_request(330, 300)
			if self.full_window_flag == "on":
				self.set_size_request(720, 400)
		else:
			if self.graphics_style == "info":
				#self.set_size_request(330, 190)
				self.set_size_request(330, 140)
			if self.graphics_style == "bar":
				self.set_size_request(330, 190)
			if self.graphics_style == "sphere":
				self.set_size_request(330, 370)
			if self.graphics_style == "dome":
				self.set_size_request(330, 370)
			if self.graphics_style == "moon":
				self.set_size_request(300, 220)
			if self.graphics_style == "earth":
				self.set_size_request(400, 220)
			if self.graphics_style == "horizon":
				self.set_size_request(400, 220)
			if self.graphics_style == "pole":
				self.set_size_request(330, 370)
			if self.full_window_flag == "on":
				self.set_size_request(720, 400)

		return

	def screen_changed(self, widget, old_screen=None):
		# To check if the display supports alpha channels, get the colormap
		screen = self.get_screen()
		colormap = screen.get_rgba_colormap()
		if colormap == None:
			colormap = screen.get_rgb_colormap()
			self.supports_alpha = False
		else:
			self.supports_alpha = True
    
		# Now we have a colormap appropriate for the screen, use it
		self.set_colormap(colormap)
    
		return False
	
	def unrealize(self, widget, date=None):
		# cancel timeout
		if self.timer:
			v = gobject.source_remove(self.timer)
			self.timer = None

	def set_timer(self, widget, on):
		# when called first time from __init__ widget is None
		if self.timer != None:
			gobject.source_remove(self.timer)
		if on:
			delay = 60000 
			self.timer = gobject.timeout_add(delay, self.update)
			# repaint immediately when coming to the foreground
			self.update()

	def update(self):
		self.queue_draw()
		return True

def hd_plugin_get_objects():
	global local
	global graphics_style_all 
	global graphics_style_pos2
	global graphics_style
	global text_flag
	global gps_mode
	global gps_timeout_secs
	global gps_time_flag
	global startup_lat
	global startup_long
	global planet_mode
	global planet_flag
	global star_mode
	global star_flag
	global city_flag
	global astro_flag
	global astro_sign
	global trails_flag
	global full_window_flag
	global clock_flag
	global sky_grid_flag
	global earth_grid_flag
	global moon_grid_flag
	global planet_label_flag
	global star_label_flag
	global city_label_flag
	global reset_flag
	global event_x
	global event_y
	global prev_event_x
	global prev_event_y
	global supports_alpha
	global gps_lock
	global click_count

	config = ConfigParser.ConfigParser()
	ini_file = os.path.expanduser("~") + "/.mchronos/mchronos.ini"
	config.read(ini_file)
	startup_long = config.get("location","longitude")
	startup_lat = config.get("location","latitude")
	gps_mode = config.get("gps","mode")
	gps_timeout_secs = config.get("gps","timeout")
	gps_time_flag = config.get("gps","time")
	graphics_style_all = config.get("style","graphics").split()
	text_flag = config.get("style","text")
	full_window_flag = config.get("style","full")
	clock_flag = config.get("style","clock")
	astro_flag = config.get("style","astro")
	trails_flag = config.get("style","trails")
	sky_grid_flag = config.get("style","skygrid")
	earth_grid_flag = config.get("style","earthgrid")
	moon_grid_flag = config.get("style","moongrid")
	planet_mode = config.get("style","planets")
	planet_label_flag = config.get("style","planetlabels")
	star_mode = config.get("style","stars")
	star_label_flag = config.get("style","starlabels")
	city_flag = config.get("style","cities")
	city_label_flag = config.get("style","citylabels")

	graphics_style_pos2 = 0 # test
	graphics_style = graphics_style_all[graphics_style_pos2] # test

	local = ephem.Observer()
	local.long = startup_long
	local.lat = startup_lat

	reset_flag = "off"

	if astro_flag == "on":
		astro_sign = -1
	else:
		astro_sign = 1

	#if planet_mode in ["off", "on"]:
	#	planet_flag = planet_mode

	#if star_mode in ["off", "on"]:
	#	star_flag = star_mode

	planet_flag = planet_mode
	star_flag = star_mode

	event_x = 0
	event_y = 0

	prev_event_x = 0
	prev_event_y = 0

	supports_alpha = False
	gps_lock = False

	click_count = 0

	plugin = mchronos()
	return [plugin]
